@@ -51,12 +51,12 @@ typedef  unsigned long  int  ub4;   /* unsigned 4-byte quantities */
 typedef  unsigned       char ub1;
 
 static GCC_INLINE ub4 perl_hash_function(
-    register ub1 *s_ptr,        /* the key */
-    register ub4  length        /* the length of the key */
+    register const ub1 *s_ptr,        /* the key */
+    register const ub4  length        /* the length of the key */
     )
 {
     register ub4  hash_value_int = 0;
-    register ub1 * const s_end = s_ptr+length;
+    register const ub1 * const s_end = s_ptr+length;
 
     while (s_ptr < s_end)
     {
@@ -625,15 +625,18 @@ fcs_bool_t fc_solve_check_and_add_state(
         JHSI(PValue, instance->judy_array, new_state_key, sizeof(*new_state_key));
 
         /* later_todo : Handle out-of-memory. */
-        if ((is_state_new = (*PValue == 0)))
+        if (*PValue == 0)
         {
             /* A new state. */
-            *PValue = (PWord_t)(existing_state_val = new_state_val);
+            *PValue = (PWord_t)(FCS_STATE_kv_to_collectible(new_state));
+            ON_STATE_NEW();
+            return TRUE;
         }
         else
         {
             /* Already exists. */
-            existing_state_val = (fcs_state_extra_info_t *)(*PValue);
+            FCS_STATE_collectible_to_kv(existing_state_raw, (fcs_collectible_state_t *)(*PValue));
+            return FALSE;
         }
     }
 #else
