@@ -32,6 +32,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
+#ifdef _WIN32
+#include <windows.h>
+#endif
 
 #include "set_weights.h"
 #include "fcs_user.h"
@@ -44,6 +47,16 @@
 
 #include "cmd_line_enum.h"
 
+static GCC_INLINE void nullify_newline(char * const line)
+{
+    char * const s = strchr(line, '\n');
+
+    if (s)
+    {
+        (*s) = '\0';
+    }
+}
+
 static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_man, char * * opened_files_dir_to_assign, const char * user_preset_dir)
 {
     int ret_code = 1;
@@ -79,8 +92,61 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
     }
     env_var_presetrc = getenv("FREECELL_SOLVER_PRESETRC");
 
-    global_presetrc = (FREECELL_SOLVER_PKG_DATA_DIR "/presetrc");
+#ifdef _WIN32
 
+#if 0
+#define MYDEBUG
+#endif
+
+    char windows_exe_dir[MAX_PATH] = "";
+    char windows_global_presetc[MAX_PATH + 100] = "";
+
+    /* Will contain exe path */
+    const HMODULE hModule = GetModuleHandle(NULL);
+    if (hModule != NULL)
+    {
+
+#ifdef MYDEBUG
+        fprintf(stderr, "Yomza\n");
+#endif
+        /* When passing NULL to GetModuleHandle, it returns handle of exe itself */
+        GetModuleFileName(hModule, windows_exe_dir, (sizeof(windows_exe_dir)));
+#ifdef MYDEBUG
+        fprintf(stderr, "bef--klok windows_exe_dir=<<%s>>\n", windows_exe_dir);
+#endif
+        {
+            /* Remove the basename and keep only the dirname. */
+            char * s = windows_exe_dir + strlen(windows_exe_dir) - 1;
+            while ((s > windows_exe_dir) && (!((*s == '/') || (*s == '\\'))))
+            {
+                s--;
+            }
+            if (s > windows_exe_dir)
+            {
+                *s = '\0';
+            }
+         }
+#ifdef MYDEBUG
+        fprintf(stderr, "after--klok windows_exe_dir=<<%s>>\n", windows_exe_dir);
+#endif
+         sprintf(
+             windows_global_presetc,
+             "%s/../share/freecell-solver/presetrc",
+             windows_exe_dir
+         );
+         global_presetrc = windows_global_presetc;
+#ifdef MYDEBUG
+         fprintf(stderr, "Absonom windows_global_presetc=<<%s>>\n", windows_global_presetc);
+#endif
+    }
+    else
+    {
+        global_presetrc = NULL;
+    }
+
+#else
+    global_presetrc = (FREECELL_SOLVER_PKG_DATA_DIR "/presetrc");
+#endif
     const char * path;
     fcs_bool_t read_next_preset = FALSE;
     for (int path_idx=0;(presetrc_pathes[path_idx] != NULL) ; path_idx++)
@@ -104,27 +170,43 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
             }
             if (!strncmp(line, "dir=", 4))
             {
-#define nullify_newline() \
-                {     \
-                char * s;   \
-   \
-                s = strchr(line, '\n');      \
-                if (s != NULL)     \
-                {       \
-                    *s = '\0';      \
-                }        \
-                }
-                nullify_newline();
+                nullify_newline(line);
 
                 if (opened_files_dir != NULL)
                 {
                     free(opened_files_dir);
                 }
+#ifdef _WIN32
+                {
+                    char * s = line+4;
+                    const char * const var_prefix = "${EXE_DIRNAME}";
+                    const int var_prefix_len = strlen(var_prefix);
+#if 0
+#define MYDEBUG
+#endif
+                    if (! strncmp(s, var_prefix, var_prefix_len))
+                    {
+#ifdef MYDEBUG
+                        fprintf(stderr, "Qlokoknh windows_exe_dir=<<%s>>\n", windows_exe_dir);
+#endif
+                        opened_files_dir = malloc(strlen(s) + strlen(windows_exe_dir) + 100);
+                        sprintf(
+                            opened_files_dir, "%s%s",
+                            windows_exe_dir, s+var_prefix_len
+                        );
+                    }
+                    else
+                    {
+                        opened_files_dir = strdup(s);
+                    }
+                }
+#else
                 opened_files_dir = strdup(line+4);
+#endif
             }
             else if (!strncmp(line, "name=", 5))
             {
-                nullify_newline();
+                nullify_newline(line);
                 if (!strcmp(line+5, preset_name))
                 {
                     read_next_preset = TRUE;
@@ -147,7 +229,6 @@ static GCC_INLINE int read_preset(const char * preset_name, args_man_t * * args_
         }
         fclose(f);
         f = NULL;
-#undef nullify_newline
     }
 have_preset:
 
@@ -289,8 +370,7 @@ case '-':
 { switch(*(p++)) {
 case 'a':
 {
-if (!strncmp(p, "-star-weights", 13)) {
-p += 13;
+if (!strcmp(p, "-star-weights")) {
 opt = FCS_OPT_BEFS_WEIGHTS;
 
 }
@@ -306,8 +386,7 @@ if (*(p++) == 'a')
 { switch(*(p++)) {
 case 'c':
 {
-if (!strncmp(p, "he-limit", 8)) {
-p += 8;
+if (!strcmp(p, "he-limit")) {
 opt = FCS_OPT_CACHE_LIMIT;
 
 }
@@ -317,8 +396,7 @@ break;
 
 case 'l':
 {
-if (!strncmp(p, "c-real-depth", 12)) {
-p += 12;
+if (!strcmp(p, "c-real-depth")) {
 opt = FCS_OPT_CALC_REAL_DEPTH;
 
 }
@@ -343,8 +421,7 @@ if (*(p++) == 'e')
 { switch(*(p++)) {
 case 'c':
 {
-if (!strncmp(p, "ks-num", 6)) {
-p += 6;
+if (!strcmp(p, "ks-num")) {
 opt = FCS_OPT_DECKS_NUM;
 
 }
@@ -354,8 +431,7 @@ break;
 
 case 'p':
 {
-if (!strncmp(p, "th-tests-order", 14)) {
-p += 14;
+if (!strcmp(p, "th-tests-order")) {
 opt = FCS_OPT_DEPTH_TESTS_ORDER;
 
 }
@@ -374,8 +450,7 @@ break;
 
 case 'e':
 {
-if (!strncmp(p, "mpty-stacks-filled-by", 21)) {
-p += 21;
+if (!strcmp(p, "mpty-stacks-filled-by")) {
 opt = FCS_OPT_EMPTY_STACKS_FILLED_BY;
 
 }
@@ -392,8 +467,7 @@ p += 3;
 { switch(*(p++)) {
 case '-':
 {
-if (!strncmp(p, "name", 4)) {
-p += 4;
+if (!strcmp(p, "name")) {
 opt = FCS_OPT_FLARE_NAME;
 
 }
@@ -409,8 +483,7 @@ if (*(p++) == '-')
 { switch(*(p++)) {
 case 'c':
 {
-if (!strncmp(p, "hoice", 5)) {
-p += 5;
+if (!strcmp(p, "hoice")) {
 opt = FCS_OPT_FLARES_CHOICE;
 
 }
@@ -420,8 +493,7 @@ break;
 
 case 'i':
 {
-if (!strncmp(p, "ters-factor", 11)) {
-p += 11;
+if (!strcmp(p, "ters-factor")) {
 opt = FCS_OPT_FLARES_ITERS_FACTOR;
 
 }
@@ -431,8 +503,7 @@ break;
 
 case 'p':
 {
-if (!strncmp(p, "lan", 3)) {
-p += 3;
+if (!strcmp(p, "lan")) {
 opt = FCS_OPT_FLARES_PLAN;
 
 }
@@ -459,8 +530,7 @@ break;
 
 case 'r':
 {
-if (!strncmp(p, "eecells-num", 11)) {
-p += 11;
+if (!strcmp(p, "eecells-num")) {
 opt = FCS_OPT_FREECELLS_NUM;
 
 }
@@ -475,8 +545,7 @@ break;
 
 case 'g':
 {
-if (!strncmp(p, "ame", 3)) {
-p += 3;
+if (!strcmp(p, "ame")) {
 opt = FCS_OPT_GAME;
 
 }
@@ -486,8 +555,7 @@ break;
 
 case 'l':
 {
-if (!strncmp(p, "oad-config", 10)) {
-p += 10;
+if (!strcmp(p, "oad-config")) {
 opt = FCS_OPT_LOAD_CONFIG;
 
 }
@@ -504,8 +572,7 @@ p += 2;
 { switch(*(p++)) {
 case 'd':
 {
-if (!strncmp(p, "epth", 4)) {
-p += 4;
+if (!strcmp(p, "epth")) {
 opt = FCS_OPT_MAX_DEPTH;
 
 }
@@ -515,8 +582,7 @@ break;
 
 case 'i':
 {
-if (!strncmp(p, "ters", 4)) {
-p += 4;
+if (!strcmp(p, "ters")) {
 opt = FCS_OPT_MAX_ITERS;
 
 }
@@ -526,8 +592,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "tored-states", 12)) {
-p += 12;
+if (!strcmp(p, "tored-states")) {
 opt = FCS_OPT_MAX_STORED_STATES;
 
 }
@@ -545,8 +610,7 @@ break;
 
 case 'e':
 {
-if (!strncmp(p, "thod", 4)) {
-p += 4;
+if (!strcmp(p, "thod")) {
 opt = FCS_OPT_METHOD;
 
 }
@@ -566,8 +630,7 @@ p += 4;
 { switch(*(p++)) {
 case 'f':
 {
-if (!strncmp(p, "lare", 4)) {
-p += 4;
+if (!strcmp(p, "lare")) {
 opt = FCS_OPT_NEXT_FLARE;
 
 }
@@ -577,8 +640,7 @@ break;
 
 case 'h':
 {
-if (!strncmp(p, "ard-thread", 10)) {
-p += 10;
+if (!strcmp(p, "ard-thread")) {
 opt = FCS_OPT_NEXT_HARD_THREAD;
 
 }
@@ -588,8 +650,7 @@ break;
 
 case 'i':
 {
-if (!strncmp(p, "nstance", 7)) {
-p += 7;
+if (!strcmp(p, "nstance")) {
 opt = FCS_OPT_NEXT_INSTANCE;
 
 }
@@ -599,8 +660,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "oft-thread", 10)) {
-p += 10;
+if (!strcmp(p, "oft-thread")) {
 opt = FCS_OPT_NEXT_SOFT_THREAD;
 
 }
@@ -623,8 +683,7 @@ p += 6;
 { switch(*(p++)) {
 case 'a':
 {
-if (!strncmp(p, "tion-tests-order", 16)) {
-p += 16;
+if (!strcmp(p, "tion-tests-order")) {
 opt = FCS_OPT_OPTIMIZATION_TESTS_ORDER;
 
 }
@@ -634,8 +693,7 @@ break;
 
 case 'e':
 {
-if (!strncmp(p, "-solution", 9)) {
-p += 9;
+if (!strcmp(p, "-solution")) {
 opt = FCS_OPT_OPTIMIZE_SOLUTION;
 
 }
@@ -658,8 +716,7 @@ p += 2;
 { switch(*(p++)) {
 case 'l':
 {
-if (!strncmp(p, "ude", 3)) {
-p += 3;
+if (!strcmp(p, "ude")) {
 opt = FCS_OPT_PRELUDE;
 
 }
@@ -669,8 +726,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "et", 2)) {
-p += 2;
+if (!strcmp(p, "et")) {
 opt = FCS_OPT_GAME;
 
 }
@@ -694,8 +750,7 @@ if (*(p++) == 'e')
 { switch(*(p++)) {
 case 'a':
 {
-if (!strncmp(p, "d-from-file", 11)) {
-p += 11;
+if (!strcmp(p, "d-from-file")) {
 opt = FCS_OPT_READ_FROM_FILE;
 
 }
@@ -705,8 +760,7 @@ break;
 
 case 'p':
 {
-if (!strncmp(p, "arent-states", 12)) {
-p += 12;
+if (!strcmp(p, "arent-states")) {
 opt = FCS_OPT_REPARENT_STATES;
 
 }
@@ -716,8 +770,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "et", 2)) {
-p += 2;
+if (!strcmp(p, "et")) {
 opt = FCS_OPT_RESET;
 
 }
@@ -738,8 +791,7 @@ case 's':
 { switch(*(p++)) {
 case 'c':
 {
-if (!strncmp(p, "ans-synergy", 11)) {
-p += 11;
+if (!strcmp(p, "ans-synergy")) {
 opt = FCS_OPT_SCANS_SYNERGY;
 
 }
@@ -756,8 +808,7 @@ p += 5;
 { switch(*(p++)) {
 case '-':
 {
-if (!strncmp(p, "move", 4)) {
-p += 4;
+if (!strcmp(p, "move")) {
 opt = FCS_OPT_SEQUENCE_MOVE;
 
 }
@@ -767,8 +818,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "-are-built-by", 13)) {
-p += 13;
+if (!strcmp(p, "-are-built-by")) {
 opt = FCS_OPT_SEQUENCES_ARE_BUILT_BY;
 
 }
@@ -786,8 +836,7 @@ break;
 
 case 't':
 {
-if (!strncmp(p, "-pruning", 8)) {
-p += 8;
+if (!strcmp(p, "-pruning")) {
 opt = FCS_OPT_SET_PRUNING;
 
 }
@@ -802,8 +851,7 @@ break;
 
 case 'o':
 {
-if (!strncmp(p, "ft-thread-step", 14)) {
-p += 14;
+if (!strcmp(p, "ft-thread-step")) {
 opt = FCS_OPT_SOFT_THREAD_STEP;
 
 }
@@ -815,8 +863,7 @@ case 't':
 { switch(*(p++)) {
 case '-':
 {
-if (!strncmp(p, "name", 4)) {
-p += 4;
+if (!strcmp(p, "name")) {
 opt = FCS_OPT_ST_NAME;
 
 }
@@ -826,8 +873,7 @@ break;
 
 case 'a':
 {
-if (!strncmp(p, "cks-num", 7)) {
-p += 7;
+if (!strcmp(p, "cks-num")) {
 opt = FCS_OPT_STACKS_NUM;
 
 }
@@ -849,8 +895,7 @@ case 't':
 { switch(*(p++)) {
 case 'e':
 {
-if (!strncmp(p, "sts-order", 9)) {
-p += 9;
+if (!strcmp(p, "sts-order")) {
 opt = FCS_OPT_TESTS_ORDER;
 
 }
@@ -860,8 +905,7 @@ break;
 
 case 'r':
 {
-if (!strncmp(p, "im-max-stored-states", 20)) {
-p += 20;
+if (!strcmp(p, "im-max-stored-states")) {
 opt = FCS_OPT_TRIM_MAX_STORED_STATES;
 
 }
@@ -881,8 +925,7 @@ break;
 
 case 'a':
 {
-if (!strncmp(p, "sw", 2)) {
-p += 2;
+if (!strcmp(p, "sw")) {
 opt = FCS_OPT_BEFS_WEIGHTS;
 
 }
@@ -892,8 +935,7 @@ break;
 
 case 'd':
 {
-if (!strncmp(p, "to", 2)) {
-p += 2;
+if (!strcmp(p, "to")) {
 opt = FCS_OPT_DEPTH_TESTS_ORDER;
 
 }
@@ -903,8 +945,7 @@ break;
 
 case 'f':
 {
-if (!strncmp(p, "if", 2)) {
-p += 2;
+if (!strcmp(p, "if")) {
 opt = FCS_OPT_FLARES_ITERS_FACTOR;
 
 }
@@ -976,8 +1017,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "s", 1)) {
-p += 1;
+if (!strcmp(p, "s")) {
 opt = FCS_OPT_MAX_STORED_STATES;
 
 }
@@ -1006,8 +1046,7 @@ break;
 
 case 'h':
 {
-if (!strncmp(p, "t", 1)) {
-p += 1;
+if (!strcmp(p, "t")) {
 opt = FCS_OPT_NEXT_HARD_THREAD;
 
 }
@@ -1029,8 +1068,7 @@ break;
 
 case 's':
 {
-if (!strncmp(p, "t", 1)) {
-p += 1;
+if (!strcmp(p, "t")) {
 opt = FCS_OPT_NEXT_SOFT_THREAD;
 
 }
@@ -1057,8 +1095,7 @@ break;
 
 case '-':
 {
-if (!strncmp(p, "to", 2)) {
-p += 2;
+if (!strcmp(p, "to")) {
 opt = FCS_OPT_OPTIMIZATION_TESTS_ORDER;
 
 }
@@ -1078,8 +1115,7 @@ case 's':
 { switch(*(p++)) {
 case 'e':
 {
-if (!strncmp(p, "ed", 2)) {
-p += 2;
+if (!strcmp(p, "ed")) {
 opt = FCS_OPT_SEED;
 
 }
@@ -1101,8 +1137,7 @@ break;
 
 case 't':
 {
-if (!strncmp(p, "ep", 2)) {
-p += 2;
+if (!strcmp(p, "ep")) {
 opt = FCS_OPT_SOFT_THREAD_STEP;
 
 }
@@ -1119,8 +1154,7 @@ case 't':
 { switch(*(p++)) {
 case 'm':
 {
-if (!strncmp(p, "ss", 2)) {
-p += 2;
+if (!strcmp(p, "ss")) {
 opt = FCS_OPT_TRIM_MAX_STORED_STATES;
 
 }
