@@ -38,12 +38,18 @@ extern "C"
 
 #include "dbm_common.h"
 
+typedef struct
+{
+    unsigned char s[RANK_KING];
+} fcs_which_moves_bitmask_t;
+
 typedef struct
 {
     char * state_string;
     unsigned char move;
     int core_irreversible_moves_count;
     int num_non_reversible_moves_including_prune;
+    fcs_which_moves_bitmask_t which_irreversible_moves_bitmask;
 } fcs_derived_state_debug_t;
 
 DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
