@@ -98,7 +98,6 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
     instance_solver_thread_calc_derived_states(
         local_variant,
         &init_state,
-        &enc_state,
         NULL,
         &derived_list,
         &derived_list_recycle_bin,
@@ -130,7 +129,6 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
         debug_ret[idx].state_string =
         fc_solve_state_as_string(
             &(iter->state.s),
-            &(iter->state.info),
             &locs,
             FREECELLS_NUM,
             STACKS_NUM,
@@ -144,6 +142,9 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
             = iter->core_irreversible_moves_count;
         debug_ret[idx].num_non_reversible_moves_including_prune
             = iter->num_non_reversible_moves_including_prune;
+        /* TODO : Put something meaningful there by passing it to the function. */
+        debug_ret[idx].which_irreversible_moves_bitmask =
+            iter->which_irreversible_moves_bitmask;
         idx++;
         iter = iter->next;
     }
@@ -184,11 +185,13 @@ DLLEXPORT int fc_solve_user_INTERNAL_perform_horne_prune(
             indirect_stacks_buffer
             );
 
-    prune_ret = horne_prune(local_variant, &init_state, NULL, NULL);
+    {
+        fcs_which_moves_bitmask_t which_no_use = {{'\0'}};
+        prune_ret = horne_prune(local_variant, &init_state, &which_no_use, NULL, NULL);
+    }
     *ret_state_s =
         fc_solve_state_as_string(
             &(init_state.s),
-            &(init_state.info),
             &locs,
             FREECELLS_NUM,
             STACKS_NUM,
