@@ -51,7 +51,8 @@ dict_t * fc_solve_dbm_store_get_dict(
 fcs_dbm_record_t * fc_solve_dbm_store_insert_key_value(
     fcs_dbm_store_t store,
     const fcs_encoded_state_buffer_t * key,
-    fcs_dbm_record_t * parent
+    fcs_dbm_record_t * parent,
+    fcs_bool_t should_modify_parent
 )
 {
 #ifdef FCS_LIBAVL_STORE_WHOLE_KEYS
@@ -79,7 +80,7 @@ fcs_dbm_record_t * fc_solve_dbm_store_insert_key_value(
     fcs_dbm_record_set_parent_ptr(to_check, parent);
 #else
     to_check->key = *key;
-    to_check->parent = *parent;
+    to_check->parent = parent->parent;
 #endif
     fcs_bool_t ret =
         (fc_solve_kaz_tree_alloc_insert(db->kaz_tree, to_check) == NULL);
@@ -92,10 +93,12 @@ fcs_dbm_record_t * fc_solve_dbm_store_insert_key_value(
 #endif
     if (ret)
     {
-        if (parent)
+#ifdef FCS_EXPLICIT_REFCOUNT
+        if (should_modify_parent && parent)
         {
             fcs_dbm_record_increment_refcount(parent);
         }
+#endif
 
         return ((fcs_dbm_record_t *)(fc_solve_kaz_tree_lookup_value(db->kaz_tree, to_check)));
     }
@@ -105,6 +108,19 @@ fcs_dbm_record_t * fc_solve_dbm_store_insert_key_value(
     }
 }
 
+fcs_dbm_record_t * fc_solve_dbm_store_lookup_val(
+    fcs_dbm_store_t store,
+    const unsigned char * key
+)
+{
+    fcs_dbm_record_t to_check;
+    to_check.key = *(const fcs_encoded_state_buffer_t *)key;
+
+    dict_key_t existing = fc_solve_kaz_tree_lookup_value(((dbm_t *)store)->kaz_tree, &to_check);
+
+    return (fcs_dbm_record_t *)existing;
+}
+
 fcs_bool_t fc_solve_dbm_store_lookup_parent(
     fcs_dbm_store_t store,
     const unsigned char * key,
