@@ -183,11 +183,13 @@ static GCC_INLINE void instance_destroy(
 #include "dbm_procs.h"
 
 static GCC_INLINE void instance_check_key(
+    fcs_dbm_solver_thread_t * thread,
     fcs_dbm_solver_instance_t * instance,
     int key_depth,
     fcs_encoded_state_buffer_t * key,
     fcs_dbm_record_t * parent,
-    unsigned char move
+    unsigned char move,
+    fcs_which_moves_bitmask_t * which_irreversible_moves_bitmask
 #ifdef FCS_DBM_CACHE_ONLY
     , const fcs_fcc_move_t * moves_to_parent
 #endif
@@ -226,7 +228,7 @@ static GCC_INLINE void instance_check_key(
 #endif
     else
 #else
-    if ((token = fc_solve_dbm_store_insert_key_value(instance->store, key, parent)))
+    if ((token = fc_solve_dbm_store_insert_key_value(instance->store, key, parent, TRUE)))
 #endif
     {
 #ifdef FCS_DBM_CACHE_ONLY
@@ -260,10 +262,11 @@ static GCC_INLINE void instance_check_key(
 }
 
 
-typedef struct {
+struct fcs_dbm_solver_thread_struct
+{
     fcs_dbm_solver_instance_t * instance;
     fc_solve_delta_stater_t * delta_stater;
-} fcs_dbm_solver_thread_t;
+};
 
 typedef struct {
     fcs_dbm_solver_thread_t * thread;
@@ -396,7 +399,6 @@ static void * instance_run_solver_thread(void * void_arg)
         if (instance_solver_thread_calc_derived_states(
             local_variant,
             &state,
-            &(item->key),
             token,
             &derived_list,
             &derived_list_recycle_bin,
@@ -430,7 +432,7 @@ static void * instance_run_solver_thread(void * void_arg)
             );
         }
 
-        instance_check_multiple_keys(instance, derived_list
+        instance_check_multiple_keys(thread, instance, derived_list
 #ifdef FCS_DBM_CACHE_ONLY
             , item->moves_to_key
 #endif
@@ -465,7 +467,7 @@ typedef struct {
 
 #define USER_STATE_SIZE 2000
 
-static void populate_instance_with_intermediate_input_line(
+static fcs_bool_t populate_instance_with_intermediate_input_line(
     fcs_dbm_solver_instance_t * instance,
     fc_solve_delta_stater_t * delta,
     fcs_state_keyval_pair_t * init_state_ptr,
@@ -487,6 +489,10 @@ static void populate_instance_with_intermediate_input_line(
 #endif
     fcs_fcc_move_t move;
     enum fcs_dbm_variant_type_t local_variant;
+#ifdef DEBUG_OUT
+    fcs_state_locs_struct_t locs;
+    fc_solve_init_locs(&locs);
+#endif
 
     DECLARE_IND_BUF_T(running_indirect_stacks_buffer)
 
@@ -510,9 +516,15 @@ static void populate_instance_with_intermediate_input_line(
                 );
             exit(-1);
         }
+#ifdef FCS_DEBONDT_DELTA_STATES
+        final_stack_encoded_state.s[
+            (s_ptr - line) >> 1
+            ] = (unsigned char)hex_digits;
+#else
         final_stack_encoded_state.s[
             ++final_stack_encoded_state.s[0]
             ] = (unsigned char)hex_digits;
+#endif
         s_ptr += 2;
     }
 
@@ -540,12 +552,32 @@ static void populate_instance_with_intermediate_input_line(
     cache_insert(&(instance->cache), &(running_key), running_moves, '\0');
 #endif
 #else
-    running_parent = fc_solve_dbm_store_insert_key_value(instance->store, &(running_key), running_parent);
+    running_parent = fc_solve_dbm_store_insert_key_value(instance->store, &(running_key), running_parent, TRUE);
 #endif
     instance->num_states_in_collection++;
 
     while (sscanf(s_ptr, "%2X,", &hex_digits) == 1)
     {
+#ifdef DEBUG_OUT
+        {
+            char * state_str;
+            state_str = fc_solve_state_as_string(
+                &(running_state.s),
+                &(running_state.info),
+                &locs,
+                FREECELLS_NUM,
+                8,
+                1,
+                1,
+                0,
+                1
+            );
+
+            fprintf(stdout, "BEFORE_RUNNING_STATE == <<<\n%s>>>\n\n", state_str);
+            fflush(stdout);
+            free(state_str);
+        }
+#endif
         int src, dest;
         fcs_card_t src_card;
 
@@ -604,7 +636,10 @@ static void populate_instance_with_intermediate_input_line(
             }
         }
 #undef the_state
-        horne_prune(local_variant, &running_state, NULL, NULL);
+        {
+            fcs_which_moves_bitmask_t which_no_use = {""};
+            horne_prune(local_variant, &running_state, &which_no_use, NULL, NULL);
+        }
 
         fcs_init_and_encode_state(delta, local_variant, &(running_state),
                                   &(running_key));
@@ -616,7 +651,11 @@ static void populate_instance_with_intermediate_input_line(
         running_moves = (cache_insert(&(instance->cache), &(running_key), running_moves, move))->moves_to_key;
 #endif
 #else
-        token = fc_solve_dbm_store_insert_key_value(instance->store, &(running_key), running_parent);
+        token = fc_solve_dbm_store_insert_key_value(instance->store, &(running_key), running_parent, TRUE);
+        if (!token)
+        {
+            return FALSE;
+        }
 #endif
         instance->num_states_in_collection++;
 
@@ -630,6 +669,26 @@ static void populate_instance_with_intermediate_input_line(
             &running_state,
             running_indirect_stacks_buffer
             );
+#ifdef DEBUG_OUT
+        {
+            char * state_str;
+            state_str = fc_solve_state_as_string(
+                &(running_state.s),
+                &(running_state.info),
+                &locs,
+                FREECELLS_NUM,
+                8,
+                1,
+                1,
+                0,
+                1
+            );
+
+            fprintf(stdout, "RUNNING_STATE == <<<\n%s>>>\n\n", state_str);
+            fflush(stdout);
+            free(state_str);
+        }
+#endif
     }
 
     if (memcmp(&running_key, &final_stack_encoded_state,
@@ -644,7 +703,7 @@ static void populate_instance_with_intermediate_input_line(
     fcs_offloading_queue__insert(&(instance->queue), (const fcs_offloading_queue_item_t *)(&token));
     instance->count_of_items_in_queue++;
 
-    return;
+    return TRUE;
 }
 
 static void instance_run_all_threads(
@@ -791,7 +850,6 @@ static unsigned char get_move_from_parent_to_child(
     instance_solver_thread_calc_derived_states(
         local_variant,
         &parent_state,
-        &parent,
         NULL,
         &derived_list,
         &derived_list_recycle_bin,
@@ -876,7 +934,6 @@ static void trace_solution(
         state_as_str =
             fc_solve_state_as_string(
                 &(state.s),
-                &(state.info),
                 &locs,
                 FREECELLS_NUM,
                 STACKS_NUM,
@@ -908,6 +965,9 @@ static fcs_bool_t handle_and_destroy_instance_solution(
 {
     fcs_bool_t ret = FALSE;
     fcs_dbm_record_t * token;
+#ifdef DEBUG_OUT
+    enum fcs_dbm_variant_type_t local_variant = instance->variant;
+#endif
 
     TRACE0("handle_and_destroy_instance_solution start");
     instance_print_stats(instance, out_fh);
@@ -933,10 +993,17 @@ static fcs_bool_t handle_and_destroy_instance_solution(
                 int i;
                 physical_item.key = token->key;
 
+#ifdef FCS_DEBONDT_DELTA_STATES
+                for (i=0; i < sizeof(item->key) ; i++)
+                {
+                    fprintf(out_fh, "%.2X", (int)item->key.s[i]);
+                }
+#else
                 for (i=0; i < item->key.s[0] ; i++)
                 {
                     fprintf(out_fh, "%.2X", (int)item->key.s[1 + i]);
                 }
+#endif
 
                 fprintf (out_fh, "%s", "|");
                 fflush(out_fh);
@@ -1226,7 +1293,10 @@ int main(int argc, char * argv[])
         init_indirect_stacks_buffer
     );
 
-    horne_prune(local_variant, &init_state, NULL, NULL);
+    {
+        fcs_which_moves_bitmask_t which_no_use = {{'\0'}};
+        horne_prune(local_variant, &init_state, &which_no_use, NULL, NULL);
+    }
 
     delta = fc_solve_delta_stater_alloc(
             &init_state.s,
@@ -1292,14 +1362,17 @@ int main(int argc, char * argv[])
                 skip_queue_output = FALSE;
                 {
                     fcs_encoded_state_buffer_t parent_state_enc;
-                    populate_instance_with_intermediate_input_line(
+                    if (!populate_instance_with_intermediate_input_line(
                         &limit_instance,
                         delta,
                         &init_state,
                         line,
                         line_num,
                         &parent_state_enc
-                        );
+                        ))
+                    {
+                        continue;
+                    }
 
                     instance_run_all_threads(
                         &limit_instance, &init_state, NUM_THREADS()
@@ -1334,14 +1407,17 @@ int main(int argc, char * argv[])
                 if (!skip_queue_output)
                 {
                     fcs_encoded_state_buffer_t parent_state_enc;
-                    populate_instance_with_intermediate_input_line(
+                    if (!populate_instance_with_intermediate_input_line(
                         &queue_instance,
                         delta,
                         &init_state,
                         line,
                         line_num,
                         &parent_state_enc
-                        );
+                        ))
+                    {
+                        continue;
+                    }
 
                     instance_run_all_threads(
                         &queue_instance, &init_state, NUM_THREADS()
@@ -1404,7 +1480,7 @@ int main(int argc, char * argv[])
         cache_insert(&(instance.cache), KEY_PTR(), NULL, '\0');
 #endif
 #else
-        token = fc_solve_dbm_store_insert_key_value(instance.store, KEY_PTR(), NULL);
+        token = fc_solve_dbm_store_insert_key_value(instance.store, KEY_PTR(), NULL, TRUE);
 #endif
 
         fcs_offloading_queue__insert(&(instance.queue), (fcs_offloading_queue_item_t *)&token);
