@@ -212,11 +212,13 @@ static GCC_INLINE void instance_destroy(
 #include "dbm_procs.h"
 
 static GCC_INLINE void instance_check_key(
+    fcs_dbm_solver_thread_t * thread,
     fcs_dbm_solver_instance_t * instance,
     int key_depth,
     fcs_encoded_state_buffer_t * key,
     fcs_dbm_record_t * parent,
-    unsigned char move
+    unsigned char move,
+    fcs_which_moves_bitmask_t * which_irreversible_moves_bitmask
 #ifdef FCS_DBM_CACHE_ONLY
     , const fcs_fcc_move_t * moves_to_parent
 #endif
@@ -258,7 +260,7 @@ static GCC_INLINE void instance_check_key(
 #endif
         else
 #else
-            if ((token = fc_solve_dbm_store_insert_key_value(coll->store, key, parent)))
+            if ((token = fc_solve_dbm_store_insert_key_value(coll->store, key, parent, TRUE)))
 #endif
             {
 #ifdef FCS_DBM_CACHE_ONLY
@@ -295,11 +297,12 @@ static GCC_INLINE void instance_check_key(
 }
 
 
-typedef struct {
+struct fcs_dbm_solver_thread_struct
+{
     fcs_dbm_solver_instance_t * instance;
     fc_solve_delta_stater_t * delta_stater;
     fcs_meta_compact_allocator_t thread_meta_alloc;
-} fcs_dbm_solver_thread_t;
+};
 
 typedef struct {
     fcs_dbm_solver_thread_t * thread;
@@ -422,7 +425,6 @@ static void * instance_run_solver_thread(void * void_arg)
             char * state_str;
             state_str = fc_solve_state_as_string(
                 &(state.s),
-                &(state.info),
                 &locs,
                 FREECELLS_NUM,
                 STACKS_NUM,
@@ -441,7 +443,6 @@ static void * instance_run_solver_thread(void * void_arg)
         if (instance_solver_thread_calc_derived_states(
             local_variant,
             &state,
-            &(item->key),
             token,
             &derived_list,
             &derived_list_recycle_bin,
@@ -475,7 +476,7 @@ static void * instance_run_solver_thread(void * void_arg)
             );
         }
 
-        instance_check_multiple_keys(instance, derived_list
+        instance_check_multiple_keys(thread, instance, derived_list
 #ifdef FCS_DBM_CACHE_ONLY
             , item->moves_to_key
 #endif
@@ -739,7 +740,6 @@ static unsigned char get_move_from_parent_to_child(
     instance_solver_thread_calc_derived_states(
         local_variant,
         &parent_state,
-        &parent,
         NULL,
         &derived_list,
         &derived_list_recycle_bin,
@@ -826,7 +826,6 @@ static void trace_solution(
         state_as_str =
             fc_solve_state_as_string(
                 &(state.s),
-                &(state.info),
                 &locs,
                 FREECELLS_NUM,
                 STACKS_NUM,
@@ -1081,7 +1080,10 @@ int main(int argc, char * argv[])
         init_indirect_stacks_buffer
     );
 
-    horne_prune(local_variant, &init_state, NULL, NULL);
+    {
+        fcs_which_moves_bitmask_t which_no_use = {{'\0'}};
+        horne_prune(local_variant, &init_state, &which_no_use, NULL, NULL);
+    }
 
     delta = fc_solve_delta_stater_alloc(
             &init_state.s,
@@ -1119,7 +1121,7 @@ int main(int argc, char * argv[])
         cache_insert(&(instance.cache), KEY_PTR(), NULL, '\0');
 #endif
 #else
-        token = fc_solve_dbm_store_insert_key_value(instance.colls_by_depth[0].store, KEY_PTR(), NULL);
+        token = fc_solve_dbm_store_insert_key_value(instance.colls_by_depth[0].store, KEY_PTR(), NULL, TRUE);
 #endif
 
         fcs_offloading_queue__insert(&(instance.colls_by_depth[0].queue),
