@@ -39,7 +39,7 @@
 
 #include "fcs_hash.h"
 
-#include "alloc.h"
+#include "meta_alloc.h"
 
 #include "inline.h"
 
@@ -52,81 +52,31 @@ static GCC_INLINE void fc_solve_hash_rehash(fc_solve_hash_t * hash);
 
 
 
-void fc_solve_hash_init(
-    fc_solve_hash_t * hash,
-#ifdef FCS_INLINED_HASH_COMPARISON
-    enum FCS_INLINED_HASH_DATA_TYPE hash_type
-#else
-#ifdef FCS_WITH_CONTEXT_VARIABLE
-    int (*compare_function)(const void * key1, const void * key2, void * context),
-    void * context
-#else
-    int (*compare_function)(const void * key1, const void * key2)
-#endif
-#endif
-    )
-{
-#define HASH_WANTED_SIZE 2048
-
-    hash->size = HASH_WANTED_SIZE;
-    hash->size_bitmask = HASH_WANTED_SIZE-1;
-    hash->max_num_elems_before_resize = (HASH_WANTED_SIZE << 1);
-
-    hash->num_elems = 0;
-
-    /* Allocate a table of size entries */
-    /* Initialize all the cells of the hash table to NULL, which indicate
-       that the end of each chain is right at the start. */
-    hash->entries = (fc_solve_hash_symlink_t *)calloc(
-        HASH_WANTED_SIZE, sizeof(hash->entries[0])
-        );
-
-    hash->list_of_vacant_items = NULL;
-
-#ifdef FCS_INLINED_HASH_COMPARISON
-    hash->hash_type = hash_type;
-#else
-    hash->compare_function = compare_function;
-#ifdef FCS_WITH_CONTEXT_VARIABLE
-    hash->context = context;
-#endif
-#endif
-
-    fc_solve_compact_allocator_init(&(hash->allocator));
-
-    return;
-}
 
 fcs_bool_t fc_solve_hash_insert(
-    fc_solve_hash_t * hash,
-    void * key,
+    fc_solve_hash_t * const hash,
+    void * const key,
 #ifdef FCS_RCS_STATES
-    void * key_id,
+    void * const key_id,
 #endif
-    void * * existing_key,
-    fc_solve_hash_value_t hash_value
+    void * * const existing_key,
+    const fc_solve_hash_value_t hash_value
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
-    , fc_solve_hash_value_t secondary_hash_value
+    , const fc_solve_hash_value_t secondary_hash_value
 #endif
     )
 {
-    fc_solve_hash_symlink_t * list;
-    fc_solve_hash_symlink_item_t * item, * last_item;
-    fc_solve_hash_symlink_item_t * * item_placeholder;
 #if defined(FCS_INLINED_HASH_COMPARISON) && defined(INDIRECT_STACK_STATES)
-    enum FCS_INLINED_HASH_DATA_TYPE hash_type;
-#endif
-
-#if defined(FCS_INLINED_HASH_COMPARISON) && defined(INDIRECT_STACK_STATES)
-    hash_type = hash->hash_type;
+    const typeof(hash->hash_type) hash_type = hash->hash_type;
 #endif
     /* Get the index of the appropriate chain in the hash table */
 #define PLACE() (hash_value & (hash->size_bitmask))
 
-    list = (hash->entries + PLACE());
+    typeof(hash->entries[0]) * const list = (hash->entries + PLACE());
 
 #undef PLACE
 
+    fc_solve_hash_symlink_item_t * * item_placeholder;
     /* If first_item is non-existent */
     if (list->first_item == NULL)
     {
@@ -136,8 +86,8 @@ fcs_bool_t fc_solve_hash_insert(
     else
     {
         /* Initialize item to the chain's first_item */
-        item = list->first_item;
-        last_item = NULL;
+        fc_solve_hash_symlink_item_t * item = list->first_item;
+        fc_solve_hash_symlink_item_t * last_item = NULL;
 
 #ifdef FCS_WITH_CONTEXT_VARIABLE
 #define MY_HASH_CONTEXT_VAR    , hash->context
@@ -202,6 +152,8 @@ fcs_bool_t fc_solve_hash_insert(
         item_placeholder = &(last_item->next);
     }
 
+    fc_solve_hash_symlink_item_t * item;
+
     if (hash->list_of_vacant_items)
     {
         hash->list_of_vacant_items = (item = hash->list_of_vacant_items)->next;
@@ -222,7 +174,7 @@ fcs_bool_t fc_solve_hash_insert(
 #endif
     item->next = NULL;
 
-    if ((++hash->num_elems) > hash->max_num_elems_before_resize)
+    if ((++(hash->num_elems)) > hash->max_num_elems_before_resize)
     {
         fc_solve_hash_rehash(hash);
     }
@@ -238,35 +190,37 @@ fcs_bool_t fc_solve_hash_insert(
 
   */
 static GCC_INLINE void fc_solve_hash_rehash(
-    fc_solve_hash_t * hash
+    fc_solve_hash_t * const hash
     )
 {
-    int old_size, new_size, new_size_bitmask;
-    int i;
-    fc_solve_hash_symlink_item_t * item, * next_item;
-    int place;
-    fc_solve_hash_symlink_t * new_entries;
+    const int old_size = hash->size;
 
-    old_size = hash->size;
+    const int new_size = old_size << 1;
 
-    new_size = old_size << 1;
-    new_size_bitmask = new_size - 1;
+    /* Check for overflow. */
+    if (new_size < old_size)
+    {
+        hash->max_num_elems_before_resize = FCS_INT_LIMIT_MAX;
+        return;
+    }
 
-    new_entries = calloc(new_size, sizeof(new_entries[0]));
+    const int new_size_bitmask = new_size - 1;
+
+    fc_solve_hash_symlink_t * const new_entries = calloc(new_size, sizeof(new_entries[0]));
 
     /* Copy the items to the new hash while not allocating them again */
-    for(i=0;i<old_size;i++)
+    for (int i=0 ; i < old_size ; i++)
     {
-        item = hash->entries[i].first_item;
+        fc_solve_hash_symlink_item_t * item = hash->entries[i].first_item;
         /* traverse the chain item by item */
         while(item != NULL)
         {
             /* The place in the new hash table */
-            place = item->hash_value & new_size_bitmask;
+            const int place = item->hash_value & new_size_bitmask;
 
             /* Store the next item in the linked list in a safe place,
                so we can retrieve it after the assignment */
-            next_item = item->next;
+            fc_solve_hash_symlink_item_t * const next_item = item->next;
             /* It is placed in front of the first element in the chain,
                so it should link to it */
             item->next = new_entries[place].first_item;
