@@ -39,7 +39,6 @@
 #include "config.h"
 
 #include "state.h"
-#include "card.h"
 #include "instance.h"
 #include "scans.h"
 
@@ -49,10 +48,7 @@
 
 #include "inline.h"
 #include "likely.h"
-
-#ifndef min
-#define min(a,b) (((a)<(b))?(a):(b))
-#endif
+#include "min_and_max.h"
 
 /*
  * Throughout this code the following local variables are used to quickly
@@ -189,11 +185,11 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_move_freecell_cards_to_founds)
  * into freeeclls and empty columns
  */
 static GCC_INLINE int empty_two_cols_from_new_state(
-    fc_solve_soft_thread_t * soft_thread,
-    fcs_kv_state_t * kv_ptr_new_state,
-    fcs_move_stack_t * moves,
+    const fc_solve_soft_thread_t * const soft_thread,
+    fcs_kv_state_t * const kv_ptr_new_state,
+    fcs_move_stack_t * const moves,
     const int cols_indexes[3],
-    int nc1, int nc2
+    const int nc1, const int nc2
 )
 {
 #define key_ptr_new_state_key (kv_ptr_new_state->key)
@@ -201,10 +197,7 @@ static GCC_INLINE int empty_two_cols_from_new_state(
 #define temp_new_state_key (*key_ptr_new_state_key)
     int ret = -1;
 
-    int num_cards_to_move_from_columns[3];
-    num_cards_to_move_from_columns[0] = nc1;
-    num_cards_to_move_from_columns[1] = nc2;
-    num_cards_to_move_from_columns[2] = -1;
+    int num_cards_to_move_from_columns[3] = {nc1, nc2, -1};
 
     const int * col_idx = cols_indexes;
     int * col_num_cards = num_cards_to_move_from_columns;
@@ -214,14 +207,12 @@ static GCC_INLINE int empty_two_cols_from_new_state(
 #endif
 
 #ifdef INDIRECT_STACK_STATES
-    char * indirect_stacks_buffer = soft_thread->hard_thread->indirect_stacks_buffer;
+    char * const indirect_stacks_buffer = soft_thread->hard_thread->indirect_stacks_buffer;
 #endif
 
 
     {
-        int dest_fc_idx;
-
-        dest_fc_idx = 0;
+        int dest_fc_idx = 0;
 
         while (1)
         {
@@ -237,9 +228,7 @@ static GCC_INLINE int empty_two_cols_from_new_state(
             /* Find a vacant freecell */
             for( ; dest_fc_idx < LOCAL_FREECELLS_NUM ; dest_fc_idx++)
             {
-                if (fcs_freecell_rank(
-                        temp_new_state_key, dest_fc_idx
-                    ) == 0)
+                if (fcs_freecell_is_empty(temp_new_state_key, dest_fc_idx))
                 {
                     break;
                 }
@@ -250,7 +239,7 @@ static GCC_INLINE int empty_two_cols_from_new_state(
                 break;
             }
 
-            fcs_cards_column_t new_from_which_col = fcs_state_get_col(temp_new_state_key, *col_idx);
+            const fcs_cards_column_t new_from_which_col = fcs_state_get_col(temp_new_state_key, *col_idx);
 
             fcs_card_t top_card;
             fcs_col_pop_card(new_from_which_col, top_card);
@@ -287,14 +276,10 @@ static GCC_INLINE int empty_two_cols_from_new_state(
     }
 
     {
-        int put_cards_in_col_idx;
-
-        put_cards_in_col_idx = 0;
+        int put_cards_in_col_idx = 0;
         /* Fill the free stacks with the cards below them */
         while (1)
         {
-            fcs_cards_column_t new_b_col;
-
             while ((*col_num_cards) == 0)
             {
                 col_num_cards++;
@@ -319,9 +304,12 @@ static GCC_INLINE int empty_two_cols_from_new_state(
 
             fcs_copy_stack(temp_new_state_key, *(kv_ptr_new_state->val), put_cards_in_col_idx, indirect_stacks_buffer);
 
-            new_b_col = fcs_state_get_col(temp_new_state_key, put_cards_in_col_idx);
+            const fcs_cards_column_t new_b_col
+                = fcs_state_get_col(temp_new_state_key, put_cards_in_col_idx);
 
-            fcs_cards_column_t new_from_which_col = fcs_state_get_col(temp_new_state_key, *col_idx);
+            const int col_idx_val = *col_idx;
+            const fcs_cards_column_t new_from_which_col
+                = fcs_state_get_col(temp_new_state_key, col_idx_val);
 
             fcs_card_t top_card;
             fcs_col_pop_card(new_from_which_col, top_card);
@@ -329,13 +317,13 @@ static GCC_INLINE int empty_two_cols_from_new_state(
 
             fcs_push_1card_seq(
                 moves,
-                *col_idx,
+                col_idx_val,
                 put_cards_in_col_idx
             );
 
             ret = (put_cards_in_col_idx | (1 << 8));
 
-            fcs_flip_top_card(*col_idx);
+            fcs_flip_top_card(col_idx_val);
 
             (*col_num_cards)--;
             put_cards_in_col_idx++;
@@ -1404,7 +1392,7 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_empty_stack_into_freecells)
 
                 my_copy_stack(stack_idx);
 
-                fcs_cards_column_t new_src_col = fcs_state_get_col(new_state, stack_idx);
+                const fcs_cards_column_t new_src_col = fcs_state_get_col(new_state, stack_idx);
 
                 int b = 0;
                 for (int c = 0 ; c < cards_num ; c++, b++)
@@ -1913,13 +1901,13 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_atomic_move_freecell_card_to_empty_stack)
 #endif
 
 static GCC_INLINE int calc_foundation_to_put_card_on(
-    fc_solve_soft_thread_t * soft_thread,
-    fcs_state_t * my_ptr_state,
+    const fc_solve_soft_thread_t * const soft_thread,
+    const fcs_state_t * const my_ptr_state,
     const fcs_card_t card
 )
 {
 #ifndef FCS_FREECELL_ONLY
-    fc_solve_instance_t * instance = soft_thread->hard_thread->instance;
+    const fc_solve_instance_t * const instance = soft_thread->hard_thread->instance;
 #endif
 
     tests_define_seqs_built_by();
@@ -1963,9 +1951,9 @@ static GCC_INLINE int calc_foundation_to_put_card_on(
 }
 
 extern int fc_solve_sfs_raymond_prune(
-    fc_solve_soft_thread_t * soft_thread,
-    fcs_kv_state_t * raw_ptr_state_raw,
-    fcs_collectible_state_t * * ptr_next_state_val
+    fc_solve_soft_thread_t * const soft_thread,
+    fcs_kv_state_t * const raw_ptr_state_raw,
+    fcs_collectible_state_t * * const ptr_next_state_val
 )
 {
     tests_define_accessors();
@@ -1973,19 +1961,19 @@ extern int fc_solve_sfs_raymond_prune(
     SET_GAME_PARAMS();
 #endif
 
-    fcs_derived_states_list_t derived_states_list_struct;
-    derived_states_list_struct.states = NULL;
-    derived_states_list_struct.num_states = 0;
+    fcs_derived_states_list_t derived_states_list_struct
+        = {.states = NULL, .num_states = 0};
 
     sfs_check_state_begin();
 
     int num_total_cards_moved = 0;
-    int num_cards_moved = 0;
+    int num_cards_moved;
     do {
         num_cards_moved = 0;
         for ( int stack_idx=0 ; stack_idx < LOCAL_STACKS_NUM ; stack_idx++)
         {
-            fcs_cards_column_t col = fcs_state_get_col(new_state, stack_idx);
+            const fcs_cards_column_t col
+                = fcs_state_get_col(new_state, stack_idx);
             const int cards_num = fcs_col_len(col);
 
             if (cards_num)
@@ -2045,35 +2033,32 @@ extern int fc_solve_sfs_raymond_prune(
     sfs_check_state_end();
 #undef derived_states_list
 
+    register int ret_code;
+
+    if (num_total_cards_moved)
     {
-        register int ret_code;
+        register fcs_collectible_state_t * const ptr_next_state
+            = derived_states_list_struct.states[0].state_ptr;
 
-        if (num_total_cards_moved)
-        {
-            register fcs_collectible_state_t * ptr_next_state;
+        *ptr_next_state_val = ptr_next_state;
 
-            *ptr_next_state_val
-                = ptr_next_state
-                = derived_states_list_struct.states[0].state_ptr;
+        /*
+         * Set the GENERATED_BY_PRUNING flag uncondtionally. It won't
+         * hurt if it's already there, and if it's a state that was
+         * found by other means, we still shouldn't prune it, because
+         * it is already "prune-perfect".
+         * */
+        FCS_S_VISITED(ptr_next_state) |= FCS_VISITED_GENERATED_BY_PRUNING;
 
-            /*
-             * Set the GENERATED_BY_PRUNING flag uncondtionally. It won't
-             * hurt if it's already there, and if it's a state that was
-             * found by other means, we still shouldn't prune it, because
-             * it is already "prune-perfect".
-             * */
-            FCS_S_VISITED(ptr_next_state) |= FCS_VISITED_GENERATED_BY_PRUNING;
-
-            ret_code = PRUNE_RET_FOLLOW_STATE;
-        }
-        else
-        {
-            *ptr_next_state_val = NULL;
-            ret_code = PRUNE_RET_NOT_FOUND;
-        }
-
-        free(derived_states_list_struct.states);
-
-        return ret_code;
+        ret_code = PRUNE_RET_FOLLOW_STATE;
     }
+    else
+    {
+        *ptr_next_state_val = NULL;
+        ret_code = PRUNE_RET_NOT_FOUND;
+    }
+
+    free(derived_states_list_struct.states);
+
+    return ret_code;
 }
