@@ -40,12 +40,11 @@
 #include <fcntl.h>
 #include <limits.h>
 
-#define NUM_TIMES_STEP 50
+#define NUM_CHECKED_STATES_STEP 50
 
 #include "config.h"
 
 #include "state.h"
-#include "card.h"
 #include "instance.h"
 #include "scans.h"
 
@@ -58,6 +57,7 @@
 #include "likely.h"
 #include "count.h"
 #include "alloc_wrap.h"
+#include "fcs_strndup.h"
 
 /*
     General use of this interface:
@@ -89,14 +89,11 @@ static GCC_INLINE void normalize_befs_weights(
 {
     /* Normalize the Best-First-Search Weights, so the sum of all of them would be 1. */
 
-    double sum;
+    double sum = 0;
 
-    int weight_idx;
-
-    sum = 0;
 #define my_befs_weights weighting->befs_weights
     for (
-        weight_idx = 0
+        int weight_idx = 0
             ;
         weight_idx < COUNT(my_befs_weights)
             ;
@@ -113,63 +110,56 @@ static GCC_INLINE void normalize_befs_weights(
     {
         sum = 1;
     }
-    for (weight_idx=0 ; weight_idx < COUNT(my_befs_weights) ; weight_idx++)
+    for (int weight_idx=0 ; weight_idx < COUNT(my_befs_weights) ; weight_idx++)
     {
         my_befs_weights[weight_idx] /= sum;
     }
 }
 
 static GCC_INLINE void soft_thread_clean_soft_dfs(
-    fc_solve_soft_thread_t * soft_thread
+    fc_solve_soft_thread_t * const soft_thread
 )
 {
-    int max_depth;
-    int dfs_max_depth;
-    fcs_soft_dfs_stack_item_t * soft_dfs_info, * info_ptr;
+    fcs_soft_dfs_stack_item_t * const soft_dfs_info = DFS_VAR(soft_thread, soft_dfs_info);
     /* Check if a Soft-DFS-type scan was called in the first place */
-    if (!(
-            FC_SOLVE_IS_DFS(soft_thread)
-        && DFS_VAR(soft_thread, soft_dfs_info)
-    ))
+    if (! soft_dfs_info )
     {
         /* If not - do nothing */
         return;
     }
 
-    soft_dfs_info = DFS_VAR(soft_thread, soft_dfs_info);
-    max_depth = DFS_VAR(soft_thread, depth);
-    dfs_max_depth = DFS_VAR(soft_thread, dfs_max_depth);
     /* De-allocate the Soft-DFS specific stacks */
+    const fcs_soft_dfs_stack_item_t * info_ptr = soft_dfs_info;
+    const fcs_soft_dfs_stack_item_t * const max_info_ptr
+        = info_ptr + DFS_VAR(soft_thread, depth);
+    const fcs_soft_dfs_stack_item_t * const dfs_max_info_ptr
+        = info_ptr + DFS_VAR(soft_thread, dfs_max_depth);
+
+    for (; info_ptr < max_info_ptr ; info_ptr++)
     {
-        int depth;
-        info_ptr = soft_dfs_info;
-        for(depth=0;depth<max_depth;depth++)
+        free (info_ptr->derived_states_list.states);
+        free (info_ptr->derived_states_random_indexes);
+        free (info_ptr->positions_by_rank);
+    }
+    for ( ; info_ptr < dfs_max_info_ptr ; info_ptr++)
+    {
+        if (likely(info_ptr->derived_states_list.states))
         {
             free(info_ptr->derived_states_list.states);
             free(info_ptr->derived_states_random_indexes);
-            free(info_ptr->positions_by_rank);
-            info_ptr++;
         }
-        for(;depth<dfs_max_depth;depth++)
-        {
-            if (likely(info_ptr->derived_states_list.states))
-            {
-                free(info_ptr->derived_states_list.states);
-                free(info_ptr->derived_states_random_indexes);
-            }
-            info_ptr++;
-        }
-
-        free(soft_dfs_info);
-
-        DFS_VAR(soft_thread, soft_dfs_info) = NULL;
-
-        DFS_VAR(soft_thread, dfs_max_depth) = 0;
-
     }
+
+    free(soft_dfs_info);
+
+    DFS_VAR(soft_thread, soft_dfs_info) = NULL;
+
+    DFS_VAR(soft_thread, dfs_max_depth) = 0;
 }
 
-extern void fc_solve_free_soft_thread_by_depth_test_array(fc_solve_soft_thread_t * soft_thread)
+extern void fc_solve_free_soft_thread_by_depth_test_array(
+    fc_solve_soft_thread_t * const soft_thread
+)
 {
     int depth_idx;
 
@@ -189,24 +179,14 @@ extern void fc_solve_free_soft_thread_by_depth_test_array(fc_solve_soft_thread_t
 }
 
 static GCC_INLINE void free_instance_soft_thread_callback(
-    fc_solve_soft_thread_t * soft_thread
+    fc_solve_soft_thread_t * const soft_thread
 )
 {
-    fcs_bool_t is_scan_befs_or_bfs = FALSE;
-    switch (soft_thread->method)
-    {
-        case FCS_METHOD_A_STAR:
-            fc_solve_PQueueFree(
-                    &(BEFS_VAR(soft_thread, pqueue))
-            );
-            /* Fall-through. */
-        case FCS_METHOD_BFS:
-        case FCS_METHOD_OPTIMIZE:
-            is_scan_befs_or_bfs = TRUE;
-            break;
-    }
+    fc_solve_PQueueFree(
+        &(BEFS_VAR(soft_thread, pqueue))
+    );
 
-    fc_solve_release_tests_list(soft_thread, is_scan_befs_or_bfs);
+    fc_solve_release_tests_list(soft_thread);
 
     fc_solve_free_soft_thread_by_depth_test_array(soft_thread);
 
@@ -217,31 +197,34 @@ static GCC_INLINE void free_instance_soft_thread_callback(
 }
 
 static GCC_INLINE void accumulate_tests_by_ptr(
-    int * tests_order,
-    fcs_tests_order_t * st_tests_order
+    int * const tests_order,
+    fcs_tests_order_t * const st_tests_order
 )
 {
-
-    for ( int i=0 ; i < st_tests_order->num_groups ; i++)
+    const fcs_tests_order_group_t * group_ptr = st_tests_order->groups;
+    const fcs_tests_order_group_t * const groups_end = group_ptr + st_tests_order->num_groups;
+    for ( ; group_ptr < groups_end ; group_ptr++)
     {
-        for (int j=0; j < st_tests_order->groups[i].num ; j++)
+        const int * test_ptr = group_ptr->tests;
+        const int * const tests_end = test_ptr + group_ptr->num;
+        for (; test_ptr < tests_end; test_ptr++)
         {
-            *tests_order |= (1 << (st_tests_order->groups[i].tests[j]));
+            *tests_order |= (1 << (*test_ptr));
         }
     }
 }
 
 static GCC_INLINE void accumulate_tests_order(
-    fc_solve_soft_thread_t * soft_thread,
-    void * context
+    fc_solve_soft_thread_t * const soft_thread,
+    void * const context
 )
 {
     accumulate_tests_by_ptr((int *)context, &(soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order));
 }
 
 static GCC_INLINE void determine_scan_completeness(
-    fc_solve_soft_thread_t * soft_thread,
-    void * global_tests_order
+    fc_solve_soft_thread_t * const soft_thread,
+    void * const global_tests_order
 )
 {
     int tests_order = 0;
@@ -255,16 +238,15 @@ static GCC_INLINE void determine_scan_completeness(
 
 
 void fc_solve_foreach_soft_thread(
-    fc_solve_instance_t * instance,
-    int callback_choice,
-    void * context
+    fc_solve_instance_t * const instance,
+    const int callback_choice,
+    void * const context
 )
 {
-    int ht_idx;
-    fc_solve_hard_thread_t * hard_thread;
 
-    for(ht_idx = 0 ; ht_idx<=instance->num_hard_threads; ht_idx++)
+    for (int ht_idx = 0 ; ht_idx<=instance->num_hard_threads; ht_idx++)
     {
+        fc_solve_hard_thread_t * hard_thread;
         if (ht_idx < instance->num_hard_threads)
         {
             hard_thread = &(instance->hard_threads[ht_idx]);
@@ -302,15 +284,19 @@ void fc_solve_foreach_soft_thread(
     }
 }
 
-static GCC_INLINE void clean_soft_dfs( fc_solve_instance_t * instance )
+static GCC_INLINE void clean_soft_dfs( fc_solve_instance_t * const instance )
 {
-    fc_solve_foreach_soft_thread(instance, FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS, NULL);
+    fc_solve_foreach_soft_thread (
+        instance,
+        FOREACH_SOFT_THREAD_CLEAN_SOFT_DFS,
+        NULL
+    );
 }
 
 
 static GCC_INLINE void init_soft_thread(
-    fc_solve_hard_thread_t * hard_thread,
-    fc_solve_soft_thread_t * soft_thread
+    fc_solve_hard_thread_t * const hard_thread,
+    fc_solve_soft_thread_t * const soft_thread
 )
 {
     soft_thread->hard_thread = hard_thread;
@@ -330,9 +316,11 @@ static GCC_INLINE void init_soft_thread(
 
     /* Initialize all the Soft-DFS stacks to NULL */
     DFS_VAR(soft_thread, soft_dfs_info) = NULL;
+    DFS_VAR(soft_thread, depth) = 0;
 
     DFS_VAR(soft_thread, tests_by_depth_array).num_units = 0;
     DFS_VAR(soft_thread, tests_by_depth_array).by_depth_units = NULL;
+    DFS_VAR(soft_thread, rand_seed) = 24;
 
     BEFS_M_VAR(soft_thread, tests_list) = NULL;
 
@@ -342,9 +330,20 @@ static GCC_INLINE void init_soft_thread(
 
     BEFS_M_VAR(soft_thread, befs_positions_by_rank) = NULL;
 
-    DFS_VAR(soft_thread, rand_seed) = 24;
 
-    soft_thread->num_times_step = NUM_TIMES_STEP;
+    memcpy(
+        BEFS_VAR(soft_thread, weighting.befs_weights),
+        fc_solve_default_befs_weights,
+        sizeof(fc_solve_default_befs_weights)
+    );
+
+    BEFS_VAR(soft_thread, pqueue).Elements = NULL;
+
+    BRFS_VAR(soft_thread, bfs_queue) =
+        BRFS_VAR(soft_thread, bfs_queue_last_item) =
+        NULL;
+
+    soft_thread->num_checked_states_step = NUM_CHECKED_STATES_STEP;
 
     soft_thread->by_depth_tests_order.by_depth_tests[0].tests_order = tests_order_dup(&(soft_thread->hard_thread->instance->instance_tests_order));
 
@@ -369,7 +368,7 @@ void fc_solve_instance__init_hard_thread(
     fc_solve_new_soft_thread(hard_thread);
 
     /* Set a limit on the Hard-Thread's scan. */
-    hard_thread->num_times_step = NUM_TIMES_STEP;
+    hard_thread->num_checked_states_step = NUM_CHECKED_STATES_STEP;
 
 
     hard_thread->prelude_as_string = NULL;
@@ -402,13 +401,13 @@ fc_solve_instance_t * fc_solve_alloc_instance(fcs_meta_compact_allocator_t * con
     instance->num_indirect_prev_states = 0;
 #endif
 
-    instance->num_times = 0;
+    instance->num_checked_states = 0;
 
     instance->num_states_in_collection = 0;
     instance->active_num_states_in_collection = 0;
 
-    instance->max_num_times = -1;
-    instance->effective_max_num_times = INT_MAX;
+    instance->max_num_checked_states = -1;
+    instance->effective_max_num_checked_states = INT_MAX;
     instance->max_depth = -1;
     instance->max_num_states_in_collection = -1;
     instance->effective_max_num_states_in_collection = INT_MAX;
@@ -567,8 +566,8 @@ void fc_solve_init_instance(fc_solve_instance_t * instance)
                 compile_prelude(hard_thread);
             }
         }
-        hard_thread->num_times_left_for_soft_thread =
-            hard_thread->soft_threads[0].num_times_step;
+        hard_thread->num_checked_states_left_for_soft_thread =
+            hard_thread->soft_threads[0].num_checked_states_step;
     }
 
     {
@@ -715,7 +714,7 @@ static gint fc_solve_glib_hash_stack_compare (
  * to the initial state
  * */
 extern void fc_solve_trace_solution(
-    fc_solve_instance_t * instance
+    fc_solve_instance_t * const instance
 )
 {
     fcs_internal_move_t canonize_move = fc_solve_empty_move;
@@ -765,8 +764,8 @@ extern void fc_solve_trace_solution(
 #if ((FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL2_TREE) || (FCS_STATE_STORAGE == FCS_STATE_STORAGE_KAZ_TREE))
 
 static GCC_INLINE fcs_state_t * rcs_states_get_state(
-    fc_solve_instance_t * instance,
-    fcs_collectible_state_t * state
+    fc_solve_instance_t * const instance,
+    fcs_collectible_state_t * const state
 )
 {
     return (
@@ -807,8 +806,8 @@ static int fc_solve_rcs_states_compare(const void * void_a, const void * void_b,
     fc_solve_resume_instance() repeatedly.
   */
 void fc_solve_start_instance_process_with_board(
-    fc_solve_instance_t * instance,
-    fcs_state_keyval_pair_t * init_state
+    fc_solve_instance_t * const instance,
+    fcs_state_keyval_pair_t * const init_state
 )
 {
     fcs_state_keyval_pair_t * state_copy_ptr;
@@ -1001,7 +1000,7 @@ void fc_solve_start_instance_process_with_board(
             {
                 hard_thread->prelude_idx = 0;
                 hard_thread->st_idx = hard_thread->prelude[hard_thread->prelude_idx].scan_idx;
-                hard_thread->num_times_left_for_soft_thread = hard_thread->prelude[hard_thread->prelude_idx].quota;
+                hard_thread->num_checked_states_left_for_soft_thread = hard_thread->prelude[hard_thread->prelude_idx].quota;
                 hard_thread->prelude_idx++;
             }
             else
@@ -1026,7 +1025,7 @@ void fc_solve_start_instance_process_with_board(
     This function should be called after the user has retrieved the
     results generated by the scan as it will destroy them.
   */
-void fc_solve_finish_instance( fc_solve_instance_t * instance )
+void fc_solve_finish_instance( fc_solve_instance_t * const instance )
 {
     /* De-allocate the state collection */
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
@@ -1117,11 +1116,9 @@ void fc_solve_finish_instance( fc_solve_instance_t * instance )
 }
 
 fc_solve_soft_thread_t * fc_solve_new_soft_thread(
-    fc_solve_hard_thread_t * hard_thread
+    fc_solve_hard_thread_t * const hard_thread
 )
 {
-    fc_solve_soft_thread_t * ret;
-
     /* Make sure we are not exceeding the maximal number of soft threads
      * for an instance. */
     if (hard_thread->instance->next_soft_thread_id == MAX_NUM_SCANS)
@@ -1131,14 +1128,13 @@ fc_solve_soft_thread_t * fc_solve_new_soft_thread(
 
     hard_thread->soft_threads =
         SREALLOC(hard_thread->soft_threads, hard_thread->num_soft_threads+1);
+
+    fc_solve_soft_thread_t * ret;
+
     init_soft_thread(
         hard_thread,
-        (ret = &(hard_thread->soft_threads[hard_thread->num_soft_threads]))
+        (ret = &(hard_thread->soft_threads[hard_thread->num_soft_threads++]))
     );
 
-    hard_thread->num_soft_threads++;
-
     return ret;
 }
-
-
