@@ -50,6 +50,7 @@ extern "C" {
 
 #include "indirect_buffer.h"
 #include "rand.h"
+#include "game_type_params.h"
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE) || (defined(INDIRECT_STACK_STATES) && (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBREDBLACK_TREE))
 
@@ -238,74 +239,6 @@ typedef struct
     fcs_by_depth_tests_order_t * by_depth_tests;
 } fcs_by_depth_tests_order_array_t;
 
-typedef struct {
-    /*
-     * The number of Freecells, Stacks and Foundations present in the game.
-     *
-     * freecells_num and stacks_num are variable and may be specified at
-     * the beginning of the execution of the algorithm. However, there
-     * is a maximal limit to them which is set in config.h.
-     *
-     * decks_num can be 1 or 2 .
-     * */
-
-#define SET_INSTANCE_GAME_PARAMS(inst) \
-    const fcs_game_type_params_t game_params = (inst)->game_params
-
-#define SET_GAME_PARAMS() \
-    SET_INSTANCE_GAME_PARAMS(instance)
-
-#ifndef HARD_CODED_NUM_FREECELLS
-    fcs_game_limit_t freecells_num;
-#define INSTANCE_FREECELLS_NUM (instance->game_params.freecells_num)
-#define LOCAL_FREECELLS_NUM (game_params.freecells_num)
-#else
-#define INSTANCE_FREECELLS_NUM HARD_CODED_NUM_FREECELLS
-#define LOCAL_FREECELLS_NUM HARD_CODED_NUM_FREECELLS
-#endif
-
-#ifndef HARD_CODED_NUM_STACKS
-    fcs_game_limit_t stacks_num;
-#define INSTANCE_STACKS_NUM (instance->game_params.stacks_num)
-#define LOCAL_STACKS_NUM (game_params.stacks_num)
-#else
-#define INSTANCE_STACKS_NUM HARD_CODED_NUM_STACKS
-#define LOCAL_STACKS_NUM HARD_CODED_NUM_STACKS
-#endif
-
-#ifndef HARD_CODED_NUM_DECKS
-    fcs_game_limit_t decks_num;
-#define INSTANCE_DECKS_NUM (instance->game_params.decks_num)
-#define LOCAL_DECKS_NUM (game_params.decks_num)
-#else
-#define INSTANCE_DECKS_NUM HARD_CODED_NUM_DECKS
-#define LOCAL_DECKS_NUM HARD_CODED_NUM_DECKS
-#endif
-
-#ifndef FCS_FREECELL_ONLY
-    /* sequences_are_built_by - (bits 0:1) - what two adjacent cards in the
-     * same sequence can be.
-     *
-     * empty_stacks_fill (bits 2:3) - with what cards can empty stacks be
-     * filled with.
-     *
-     * unlimited_sequence_move - (bit 4) - whether an entire sequence can be
-     * moved from one place to the other regardless of the number of
-     * unoccupied Freecells there are.
-     * */
-    fcs_game_limit_t game_flags;
-
-#define INSTANCE_GAME_FLAGS  (instance->game_params.game_flags)
-#define GET_INSTANCE_SEQUENCES_ARE_BUILT_BY(instance) \
-    ((instance)->game_params.game_flags & 0x3)
-
-#define INSTANCE_UNLIMITED_SEQUENCE_MOVE  (INSTANCE_GAME_FLAGS & (1 << 4))
-#define INSTANCE_EMPTY_STACKS_FILL   ((INSTANCE_GAME_FLAGS >> 2) & 0x3)
-
-#endif
-
-} fcs_game_type_params_t;
-
 typedef fcs_game_limit_t fcs_runtime_flags_t;
 
 #define STRUCT_CLEAR_FLAG(instance, flag) \
@@ -434,17 +367,16 @@ struct fc_solve_instance_struct
      * */
     fcs_game_type_params_t game_params;
 
-    /* The number of states that were checked by the solving algorithm.
-     * Badly named, should be renamed to num_iters or num_checked_states */
-    fcs_int_limit_t num_times;
+    /* The number of states that were checked by the solving algorithm. */
+    fcs_int_limit_t num_checked_states;
 
     /*
-     * Like max_num_times only defaults to MAX_INT if below zero so it will
+     * Like max_num_checked_states only defaults to MAX_INT if below zero so it will
      * work without checking if it's zero.
      *
      * Normally should be used instead.
      * */
-    fcs_int_limit_t effective_max_num_times, effective_max_num_states_in_collection;
+    fcs_int_limit_t effective_max_num_checked_states, effective_max_num_states_in_collection;
     fcs_int_limit_t effective_trim_states_in_collection_from;
     /*
      * tree is the balanced binary tree that is used to store and index
@@ -563,7 +495,7 @@ struct fc_solve_instance_struct
 #endif
     /*
      * Limits for the maximal depth and for the maximal number of checked
-     * states. max_num_times is useful because it enables the process to
+     * states. max_num_checked_states is useful because it enables the process to
      * stop before it consumes too much memory.
      *
      * max_depth is quite dangerous because it blocks some intermediate moves
@@ -571,7 +503,7 @@ struct fc_solve_instance_struct
      *
      * */
     int max_depth;
-    fcs_int_limit_t max_num_times;
+    fcs_int_limit_t max_num_checked_states;
     fcs_int_limit_t trim_states_in_collection_from;
 
     /*
@@ -641,25 +573,25 @@ struct fc_solve_hard_thread_struct
 
     /*
      * The hard thread count of how many states he checked himself. The
-     * instance num_times can be confusing because other threads modify it too.
+     * instance num_checked_states can be confusing because other threads modify it too.
      *
      * Thus, the soft thread switching should be done based on this variable
      * */
-    fcs_int_limit_t num_times;
+    fcs_int_limit_t num_checked_states;
 
     /*
-     * The maximal limit for num_times.
+     * The maximal limit for num_checked_states.
      * */
-    fcs_int_limit_t max_num_times;
+    fcs_int_limit_t max_num_checked_states;
 
-    fcs_int_limit_t num_times_step;
+    fcs_int_limit_t num_checked_states_step;
 
     /*
      * This is the number of iterations that still have to be done for
-     * soft_threads[st_idx]. It is reset to (st_idx+1)->num_times_step
+     * soft_threads[st_idx]. It is reset to (st_idx+1)->num_checked_states_step
      * when st_idx is incremented.
      * */
-    fcs_int_limit_t num_times_left_for_soft_thread;
+    fcs_int_limit_t num_checked_states_left_for_soft_thread;
 
     /*
      * These variables are used to compute the MD5 checksum of a state
@@ -822,7 +754,7 @@ struct fc_solve_soft_thread_struct
 
     fc_solve_seq_cards_power_type_t initial_cards_under_sequences_value;
 
-    union
+    struct
     {
         struct
         {
@@ -882,7 +814,7 @@ struct fc_solve_soft_thread_struct
         {
             char * befs_positions_by_rank;
             fc_solve_solve_for_state_test_t * tests_list, * tests_list_end;
-            union
+            struct
             {
                 struct
                 {
@@ -939,7 +871,7 @@ struct fc_solve_soft_thread_struct
     /*
      * The number of iterations with which to process this scan
      * */
-    int num_times_step;
+    int num_checked_states_step;
 
     /*
      * A malloced string that serves as an identification for the user.
@@ -1017,7 +949,7 @@ static GCC_INLINE int update_col_cards_under_sequences(
 #ifndef FCS_FREECELL_ONLY
     const int sequences_are_built_by,
 #endif
-    const fcs_cards_column_t col,
+    const fcs_const_cards_column_t col,
     int d /* One less than cards_num of col. */
 )
 {
@@ -1285,7 +1217,7 @@ extern void fc_solve_soft_thread_init_befs_or_bfs(
 static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
 {
     fc_solve_soft_thread_t * soft_thread;
-    fcs_int_limit_t num_times_started_at;
+    fcs_int_limit_t num_checked_states_started_at;
     int ret;
     fc_solve_instance_t * instance = hard_thread->instance;
     int * st_idx_ptr = &(hard_thread->st_idx);
@@ -1318,7 +1250,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
                 (hard_thread->prelude_idx < hard_thread->prelude_num_items))   \
             {      \
                 (*st_idx_ptr) = hard_thread->prelude[hard_thread->prelude_idx].scan_idx; \
-                hard_thread->num_times_left_for_soft_thread = hard_thread->prelude[hard_thread->prelude_idx].quota; \
+                hard_thread->num_checked_states_left_for_soft_thread = hard_thread->prelude[hard_thread->prelude_idx].quota; \
                 hard_thread->prelude_idx++; \
             }    \
             else       \
@@ -1327,7 +1259,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
                 {       \
                     *(st_idx_ptr) = 0;  \
                 }      \
-                hard_thread->num_times_left_for_soft_thread = hard_thread->soft_threads[*st_idx_ptr].num_times_step;  \
+                hard_thread->num_checked_states_left_for_soft_thread = hard_thread->soft_threads[*st_idx_ptr].num_checked_states_step;  \
             }
 
 
@@ -1341,12 +1273,12 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
          * Keep record of the number of iterations since this
          * thread started.
          * */
-        num_times_started_at = hard_thread->num_times;
+        num_checked_states_started_at = hard_thread->num_checked_states;
         /*
          * Calculate a soft thread-wise limit for this hard
          * thread to run.
          * */
-        hard_thread->max_num_times = hard_thread->num_times + hard_thread->num_times_left_for_soft_thread;
+        hard_thread->max_num_checked_states = hard_thread->num_checked_states + hard_thread->num_checked_states_left_for_soft_thread;
 
 
 
@@ -1358,31 +1290,20 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
          * that will abstract a scan. But it's not critical since
          * I don't support user-defined scans.
          * */
+        if (! STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED))
+        {
+            fc_solve_soft_thread_init_soft_dfs(soft_thread);
+            fc_solve_soft_thread_init_befs_or_bfs(soft_thread);
+            STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED);
+        }
         switch(soft_thread->super_method_type)
         {
             case FCS_SUPER_METHOD_DFS:
-
-            if (! STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED))
-            {
-                fc_solve_soft_thread_init_soft_dfs(soft_thread);
-                STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED);
-            }
-
             ret = fc_solve_soft_dfs_do_solve(soft_thread);
-
             break;
 
             case FCS_SUPER_METHOD_BEFS_BRFS:
-
-            if (! STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED))
-            {
-                fc_solve_soft_thread_init_befs_or_bfs(soft_thread);
-
-                STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED);
-            }
-
             ret = fc_solve_befs_or_bfs_do_solve(soft_thread);
-
             break;
 
             default:
@@ -1392,7 +1313,7 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
         /*
          * Determine how much iterations we still have left
          * */
-        hard_thread->num_times_left_for_soft_thread -= (hard_thread->num_times - num_times_started_at);
+        hard_thread->num_checked_states_left_for_soft_thread -= (hard_thread->num_checked_states - num_checked_states_started_at);
 
         /*
          * I use <= instead of == because it is possible that
@@ -1403,11 +1324,11 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
          *
          * It's a kludge, but it works.
          * */
-        if (hard_thread->num_times_left_for_soft_thread <= 0)
+        if (hard_thread->num_checked_states_left_for_soft_thread <= 0)
         {
             switch_to_next_soft_thread();
             /*
-             * Reset num_times_left_for_soft_thread
+             * Reset num_checked_states_left_for_soft_thread
              * */
 
         }
@@ -1450,10 +1371,10 @@ static GCC_INLINE int run_hard_thread(fc_solve_hard_thread_t * hard_thread)
             (
                 (ret == FCS_STATE_SUSPEND_PROCESS) &&
                 /* There's a limit to the scan only
-                 * if max_num_times is greater than 0 */
+                 * if max_num_checked_states is greater than 0 */
                 (
                     (
-                        (instance->num_times >= instance->effective_max_num_times)
+                        (instance->num_checked_states >= instance->effective_max_num_checked_states)
                     ) ||
                     (instance->num_states_in_collection >=
                         instance->effective_max_num_states_in_collection
@@ -1560,11 +1481,11 @@ static GCC_INLINE int fc_solve_optimize_solution(
     STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN);
 
     /* Initialize the optimization hard-thread and soft-thread */
-    optimization_thread->num_times_left_for_soft_thread = 1000000;
+    optimization_thread->num_checked_states_left_for_soft_thread = 1000000;
 
     /* Instruct the optimization hard thread to run indefinitely AFA it
      * is concerned */
-    optimization_thread->max_num_times = INT_MAX;
+    optimization_thread->max_num_checked_states = INT_MAX;
 
     fc_solve_soft_thread_init_befs_or_bfs(soft_thread);
     STRUCT_TURN_ON_FLAG(soft_thread, FCS_SOFT_THREAD_INITIALIZED);
@@ -1631,9 +1552,9 @@ static GCC_INLINE int fc_solve_resume_instance(
                     (
                         (ret == FCS_STATE_SUSPEND_PROCESS) &&
                         /* There's a limit to the scan only
-                         * if max_num_times is greater than 0 */
+                         * if max_num_checked_states is greater than 0 */
                         (
-                            (instance->num_times >= instance->effective_max_num_times)
+                            (instance->num_checked_states >= instance->effective_max_num_checked_states)
                             ||
                             (instance->num_states_in_collection >= instance->effective_max_num_states_in_collection)
                         )
@@ -1759,8 +1680,8 @@ static GCC_INLINE void fc_solve_reset_hard_thread(
     fc_solve_hard_thread_t * const hard_thread
 )
 {
-    hard_thread->num_times = 0;
-    hard_thread->max_num_times = INT_MAX;
+    hard_thread->num_checked_states = 0;
+    hard_thread->max_num_checked_states = INT_MAX;
     hard_thread->num_soft_threads_finished = 0;
 }
 
@@ -1773,42 +1694,37 @@ static GCC_INLINE void fc_solve_reset_soft_thread(
 }
 
 static GCC_INLINE void fc_solve_release_tests_list(
-    fc_solve_soft_thread_t * const soft_thread,
-    const fcs_bool_t is_scan_befs_or_bfs
+    fc_solve_soft_thread_t * const soft_thread
 )
 {
-    if (is_scan_befs_or_bfs)
-    {
-        free (BEFS_M_VAR(soft_thread, tests_list));
-        BEFS_M_VAR(soft_thread, tests_list) = NULL;
-    }
-    else
-    {
-        /* A DFS Scan. */
-        int unit_idx;
-        fcs_tests_by_depth_array_t * arr;
+    /* Free the BeFS data. */
+    free (BEFS_M_VAR(soft_thread, tests_list));
+    BEFS_M_VAR(soft_thread, tests_list) = NULL;
 
-        arr = &(DFS_VAR(soft_thread, tests_by_depth_array));
-        for (unit_idx = 0 ; unit_idx < arr->num_units ; unit_idx++)
+    /* Free the DFS data. */
+    int unit_idx;
+    fcs_tests_by_depth_array_t * arr;
+
+    arr = &(DFS_VAR(soft_thread, tests_by_depth_array));
+    for (unit_idx = 0 ; unit_idx < arr->num_units ; unit_idx++)
+    {
+        if (arr->by_depth_units[unit_idx].tests.lists)
         {
-            if (arr->by_depth_units[unit_idx].tests.lists)
-            {
-                fcs_tests_list_t * lists = arr->by_depth_units[unit_idx].tests.lists;
-                int num_lists = arr->by_depth_units[unit_idx].tests.num_lists;
-                int i;
+            fcs_tests_list_t * lists = arr->by_depth_units[unit_idx].tests.lists;
+            int num_lists = arr->by_depth_units[unit_idx].tests.num_lists;
+            int i;
 
-                for (i=0 ;
-                    i < num_lists ;
-                    i++)
-                {
-                    free (lists[i].tests);
-                }
-                free (lists);
+            for (i=0 ;
+                i < num_lists ;
+                i++)
+            {
+                free (lists[i].tests);
             }
+            free (lists);
         }
-        free(arr->by_depth_units);
-        arr->by_depth_units = NULL;
     }
+    free(arr->by_depth_units);
+    arr->by_depth_units = NULL;
 }
 
 static GCC_INLINE void fc_solve_instance__recycle_hard_thread(
@@ -1820,10 +1736,7 @@ static GCC_INLINE void fc_solve_instance__recycle_hard_thread(
 
     ST_LOOP_START()
     {
-        if (soft_thread->method == FCS_METHOD_A_STAR)
-        {
-            fc_solve_PQueueFree( &(BEFS_VAR(soft_thread, pqueue)) );
-        }
+        fc_solve_PQueueFree( &(BEFS_VAR(soft_thread, pqueue)) );
 
         fc_solve_reset_soft_thread(soft_thread);
     }
@@ -1839,7 +1752,7 @@ static GCC_INLINE void fc_solve_recycle_instance(
 
     fc_solve_finish_instance(instance);
 
-    instance->num_times = 0;
+    instance->num_checked_states = 0;
 
     instance->num_hard_threads_finished = 0;
 
