@@ -37,7 +37,7 @@
 #include "sfx.h"
 #endif
 
-#include "alloc.h"
+#include "meta_alloc.h"
 /*
  * Blurb for inclusion into C++ translation units
  */
@@ -130,7 +130,7 @@ typedef struct dict_load_t {
 #ifdef NO_FC_SOLVE
 extern dict_t *dict_create(dictcount_t, dict_comp_t, void * context);
 #else
-extern dict_t * fc_solve_kaz_tree_create(dict_comp_t, void * context);
+extern dict_t * fc_solve_kaz_tree_create(dict_comp_t, void * context, fcs_meta_compact_allocator_t * meta_allocator);
 #endif
 #if 0
 extern void dict_set_allocator(dict_t *, dnode_alloc_t, dnode_free_t, void *);
@@ -141,7 +141,7 @@ extern void fc_solve_kaz_tree_free_nodes(dict_t *);
 extern void dict_free(dict_t *);
 extern dict_t *dict_init(dict_t *, dictcount_t, dict_comp_t);
 #else
-extern dict_t *dict_init(dict_t *, dict_comp_t);
+extern dict_t *dict_init(dict_t *, dict_comp_t, fcs_meta_compact_allocator_t *);
 #endif
 #ifdef NO_FC_SOLVE
 extern void dict_init_like(dict_t *, const dict_t *);
