@@ -33,7 +33,7 @@
 #include <ctype.h>
 #include <assert.h>
 
-#include "card.h"
+#include "state.h"
 #include "instance.h"
 #include "preset.h"
 #include "fcs_user.h"
@@ -45,12 +45,13 @@
 #include "bool.h"
 #include "indirect_buffer.h"
 #include "count.h"
+#include "min_and_max.h"
 #include "alloc_wrap.h"
 
 #include "str_utils.h"
 
 typedef struct {
-    fcs_int_limit_t num_times;
+    fcs_int_limit_t num_checked_states;
     fcs_int_limit_t num_states_in_collection;
 } fcs_stats_t;
 
@@ -71,7 +72,7 @@ typedef struct
     int next_move;
     fcs_moves_sequence_t moves_seq;
     fcs_moves_processed_t fc_pro_moves;
-    fcs_int_limit_t obj_num_times, obj_num_states_in_collection;
+    fcs_stats_t obj_stats;
 } fcs_flare_item_t;
 
 enum
@@ -141,7 +142,7 @@ typedef struct
     /*
      * The number of iterations that the current instance started solving from.
      * */
-    fcs_stats_t init_num_times;
+    fcs_stats_t init_num_checked_states;
     /*
      * A pointer to the currently active instance out of the sequence
      * */
@@ -232,7 +233,7 @@ static void user_initialize(
     user->current_iterations_limit = -1;
 
     user->state_string_copy = NULL;
-    user->iterations_board_started_at.num_times = 0;
+    user->iterations_board_started_at.num_checked_states = 0;
     user->iterations_board_started_at.num_states_in_collection = 0;
     user->all_instances_were_suspended = TRUE;
     user->flares_choice = FLARES_CHOICE_FC_SOLVE_SOLUTION_LEN;
@@ -322,10 +323,6 @@ void DLLEXPORT freecell_solver_user_limit_current_instance_iterations(
     user->instances_list[user->current_instance_idx].limit = max_iters;
 }
 
-#ifndef min
-#define min(a,b) (((a)<(b))?(a):(b))
-#endif
-
 int DLLEXPORT freecell_solver_user_set_depth_tests_order(
     void * api_instance,
     int min_depth,
@@ -717,6 +714,31 @@ static int user_compile_all_flares_plans(
     return FCS_COMPILE_FLARES_RET_OK;
 }
 
+/*
+ * Add a trailing newline to the string if it does not exist.
+ */
+#define MYMARGIN 2
+#define TRAILING_CHAR '\n'
+static GCC_INLINE char * duplicate_string_while_adding_a_trailing_newline(
+    const char * orig_str
+)
+{
+    int len = strlen(orig_str);
+    char * s = malloc(len + MYMARGIN);
+    strcpy (s, orig_str);
+    {
+        char * s_end = s + len - 1;
+        if ((*s_end) != TRAILING_CHAR)
+        {
+            *(++s_end) = TRAILING_CHAR;
+            *(++s_end) = '\0';
+        }
+    }
+    return s;
+}
+#undef TRAILING_CHAR
+#undef MYMARGIN
+
 int DLLEXPORT freecell_solver_user_solve_board(
     void * api_instance,
     const char * state_as_string
@@ -728,7 +750,8 @@ int DLLEXPORT freecell_solver_user_solve_board(
 
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    user->state_string_copy = strdup(state_as_string);
+    user->state_string_copy =
+        duplicate_string_while_adding_a_trailing_newline(state_as_string);
 
     user->current_instance_idx = 0;
 
@@ -749,6 +772,13 @@ int DLLEXPORT freecell_solver_user_solve_board(
     return freecell_solver_user_resume_solution(api_instance);
 }
 
+static GCC_INLINE void init_stats(
+    fcs_stats_t * const s
+)
+{
+    s->num_checked_states = s->num_states_in_collection = 0;
+}
+
 static void recycle_instance(
     fcs_user_t * const user,
     const int i
@@ -775,12 +805,12 @@ static void recycle_instance(
                 flare->instance_is_ready = TRUE;
             }
             /*
-             * We have to initialize init_num_times to 0 here, because it may
-             * not get initialized again, and now the num_times of the instance
+             * We have to initialize init_num_checked_states to 0 here, because it may
+             * not get initialized again, and now the num_checked_states of the instance
              * is equal to 0.
              * */
-            user->init_num_times.num_times = 0;
-            user->init_num_times.num_states_in_collection = 0;
+            user->init_num_checked_states.num_checked_states = 0;
+            user->init_num_checked_states.num_states_in_collection = 0;
 
             flare->ret_code = FCS_STATE_NOT_BEGAN_YET;
         }
@@ -793,8 +823,7 @@ static void recycle_instance(
             flare->next_move = 0;
         }
 
-        flare->obj_num_times = 0;
-        flare->obj_num_states_in_collection = 0;
+        init_stats(&flare->obj_stats);
     }
 
     instance_item->current_plan_item_idx = 0;
@@ -917,7 +946,7 @@ int DLLEXPORT freecell_solver_user_resume_solution(
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    fcs_stats_t init_num_times;
+    fcs_stats_t init_num_checked_states;
 
     int ret = FCS_STATE_IS_NOT_SOLVEABLE;
 
@@ -961,6 +990,8 @@ int DLLEXPORT freecell_solver_user_resume_solution(
             if (instance_item->minimal_solution_flare_idx >= 0)
             {
                 user->active_flare = &(instance_item->flares[instance_item->minimal_solution_flare_idx]);
+                user->init_num_checked_states = user->active_flare->obj_stats;
+
                 ret = user->ret_code = FCS_STATE_WAS_SOLVED;
 
                 break;
@@ -1064,7 +1095,7 @@ int DLLEXPORT freecell_solver_user_resume_solution(
         }
 
 #define parameterized_fixed_limit(increment) \
-    (user->iterations_board_started_at.num_times + increment)
+    (user->iterations_board_started_at.num_checked_states + increment)
 #define parameterized_limit(increment) (((increment) < 0) ? (-1) : parameterized_fixed_limit(increment))
 #define local_limit()  \
         (instance_item->limit)
@@ -1094,19 +1125,19 @@ int DLLEXPORT freecell_solver_user_resume_solution(
 
             if (mymin < 0)
             {
-                user->active_flare->obj->max_num_times = -1;
-                user->active_flare->obj->effective_max_num_times = FCS_INT_LIMIT_MAX;
+                user->active_flare->obj->max_num_checked_states = -1;
+                user->active_flare->obj->effective_max_num_checked_states = FCS_INT_LIMIT_MAX;
             }
             else
             {
-                user->active_flare->obj->max_num_times =
-                    user->active_flare->obj->effective_max_num_times =
-                    (user->active_flare->obj->num_times + mymin - user->iterations_board_started_at.num_times);
+                user->active_flare->obj->max_num_checked_states =
+                    user->active_flare->obj->effective_max_num_checked_states =
+                    (user->active_flare->obj->num_checked_states + mymin - user->iterations_board_started_at.num_checked_states);
             }
         }
 
-        user->init_num_times.num_times = init_num_times.num_times = user->active_flare->obj->num_times;
-        user->init_num_times.num_states_in_collection = init_num_times.num_states_in_collection = user->active_flare->obj->num_states_in_collection;
+        user->init_num_checked_states.num_checked_states = init_num_checked_states.num_checked_states = user->active_flare->obj->num_checked_states;
+        user->init_num_checked_states.num_states_in_collection = init_num_checked_states.num_states_in_collection = user->active_flare->obj->num_states_in_collection;
 
         if (is_start_of_flare_solving)
         {
@@ -1133,12 +1164,11 @@ int DLLEXPORT freecell_solver_user_resume_solution(
             user->all_instances_were_suspended = FALSE;
         }
 
-        user->active_flare->obj_num_times = user->active_flare->obj->num_times;
-        user->active_flare->obj_num_states_in_collection = user->active_flare->obj->num_states_in_collection;
-        user->iterations_board_started_at.num_times += user->active_flare->obj_num_times - init_num_times.num_times;
-        user->iterations_board_started_at.num_states_in_collection += user->active_flare->obj_num_states_in_collection - init_num_times.num_states_in_collection;
-        user->init_num_times.num_times = user->active_flare->obj_num_times;
-        user->init_num_times.num_states_in_collection = user->active_flare->obj_num_states_in_collection;
+        user->active_flare->obj_stats.num_checked_states = user->active_flare->obj->num_checked_states;
+        user->active_flare->obj_stats.num_states_in_collection = user->active_flare->obj->num_states_in_collection;
+        user->iterations_board_started_at.num_checked_states += user->active_flare->obj_stats.num_checked_states - init_num_checked_states.num_checked_states;
+        user->iterations_board_started_at.num_states_in_collection += user->active_flare->obj_stats.num_states_in_collection - init_num_checked_states.num_states_in_collection;
+        user->init_num_checked_states = user->active_flare->obj_stats;
 
         if (user->ret_code == FCS_STATE_WAS_SOLVED)
         {
@@ -1174,8 +1204,8 @@ int DLLEXPORT freecell_solver_user_resume_solution(
                 flare->obj->solution_moves.moves = NULL;
             }
 
-            user->active_flare->obj_num_times = user->active_flare->obj->num_times;
-            user->active_flare->obj_num_states_in_collection = user->active_flare->obj->num_states_in_collection;
+            user->active_flare->obj_stats.num_checked_states = user->active_flare->obj->num_checked_states;
+            user->active_flare->obj_stats.num_states_in_collection = user->active_flare->obj->num_states_in_collection;
             fc_solve_recycle_instance(flare->obj);
             flare->instance_is_ready = TRUE;
 
@@ -1205,8 +1235,6 @@ int DLLEXPORT freecell_solver_user_resume_solution(
         {
             if (was_run_now)
             {
-                user->active_flare->obj_num_times = user->active_flare->obj->num_times;
-                user->active_flare->obj_num_states_in_collection = user->active_flare->obj->num_states_in_collection;
                 fc_solve_recycle_instance(user->active_flare->obj);
                 user->active_flare->instance_is_ready = TRUE;
             }
@@ -1218,7 +1246,7 @@ int DLLEXPORT freecell_solver_user_resume_solution(
              * and return now.
              * */
             if (((user->current_iterations_limit >= 0) &&
-                (user->iterations_board_started_at.num_times >=
+                (user->iterations_board_started_at.num_checked_states >=
                     user->current_iterations_limit)) ||
                 (user->active_flare->obj->num_states_in_collection >=
                     user->active_flare->obj->effective_max_num_states_in_collection)
@@ -1240,12 +1268,12 @@ int DLLEXPORT freecell_solver_user_resume_solution(
              * so, designate it as unsolvable.
              * */
             if ((local_limit() >= 0) &&
-                (user->active_flare->obj->num_times >= local_limit())
+                (user->active_flare->obj->num_checked_states >= local_limit())
                )
             {
-                user->active_flare->obj_num_times =
-                    user->active_flare->obj->num_times;
-                user->active_flare->obj_num_states_in_collection =
+                user->active_flare->obj_stats.num_checked_states =
+                    user->active_flare->obj->num_checked_states;
+                user->active_flare->obj_stats.num_states_in_collection =
                     user->active_flare->obj->num_states_in_collection;
                 recycle_instance(user, user->current_instance_idx);
                 user->current_instance_idx++;
@@ -1335,7 +1363,6 @@ DLLEXPORT char * freecell_solver_user_current_state_as_string(
         return
             fc_solve_state_as_string(
                 &(user->running_state.s),
-                &(user->running_state.info),
                 &(user->initial_state_locs),
                 INSTANCE_FREECELLS_NUM,
                 INSTANCE_STACKS_NUM,
@@ -1441,48 +1468,25 @@ void DLLEXPORT freecell_solver_user_set_solving_method(
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    user->soft_thread->super_method_type = FCS_SUPER_METHOD_BEFS_BRFS;
+    fc_solve_soft_thread_t * const soft_thread = user->soft_thread;
+    fcs_super_method_type_t super_method_type = FCS_SUPER_METHOD_BEFS_BRFS;
 
     if (method == FCS_METHOD_HARD_DFS)
     {
         method = FCS_METHOD_SOFT_DFS;
     }
 
-    switch ((user->soft_thread->method = method))
+    switch ((soft_thread->method = method))
     {
         case FCS_METHOD_RANDOM_DFS:
         case FCS_METHOD_SOFT_DFS:
         {
-            user->soft_thread->super_method_type = FCS_SUPER_METHOD_DFS;
-            DFS_VAR(user->soft_thread, dfs_max_depth)
-                = DFS_VAR(user->soft_thread, depth)
-                = DFS_VAR(user->soft_thread, tests_by_depth_array).num_units
-                = 0;
-            DFS_VAR(user->soft_thread, rand_seed) = 24;
-            DFS_VAR(user->soft_thread, soft_dfs_info) = NULL;
-            DFS_VAR(user->soft_thread, tests_by_depth_array).by_depth_units = NULL;
-        }
-        break;
-        case FCS_METHOD_A_STAR:
-        {
-            memcpy(
-                BEFS_VAR(user->soft_thread, weighting.befs_weights),
-                fc_solve_default_befs_weights,
-                sizeof(fc_solve_default_befs_weights)
-            );
-
-            BEFS_VAR(user->soft_thread, pqueue).Elements = NULL;
-        }
-        break;
-        case FCS_METHOD_OPTIMIZE:
-        case FCS_METHOD_BFS:
-        {
-            BRFS_VAR(user->soft_thread, bfs_queue) =
-            BRFS_VAR(user->soft_thread, bfs_queue_last_item) =
-            NULL;
+            super_method_type = FCS_SUPER_METHOD_DFS;
         }
         break;
     }
+
+    soft_thread->super_method_type = super_method_type;
 }
 
 #ifndef FCS_FREECELL_ONLY
@@ -1638,7 +1642,7 @@ fcs_int_limit_t DLLEXPORT freecell_solver_user_get_num_times_long(
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    return user->iterations_board_started_at.num_times + max(user->active_flare->obj_num_times, user->active_flare->obj->num_times) - user->init_num_times.num_times;
+    return user->iterations_board_started_at.num_checked_states + max(user->active_flare->obj_stats.num_checked_states, user->active_flare->obj->num_checked_states) - user->init_num_checked_states.num_checked_states;
 }
 
 int DLLEXPORT freecell_solver_user_get_num_times(void * api_instance)
@@ -1650,7 +1654,7 @@ int DLLEXPORT freecell_solver_user_get_limit_iterations(void * api_instance)
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    return user->active_flare->obj->max_num_times;
+    return user->active_flare->obj->max_num_checked_states;
 }
 
 int DLLEXPORT freecell_solver_user_get_moves_left(void * api_instance)
@@ -1739,7 +1743,7 @@ char * freecell_solver_user_get_invalid_state_error_string(
     int print_ts
     )
 {
-    char string[80], card_str[10];
+    char string[80];
 
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
@@ -1747,7 +1751,6 @@ char * freecell_solver_user_get_invalid_state_error_string(
     {
         return strdup("");
     }
-    fc_solve_card_perl2user(user->state_validity_card, card_str, print_ts);
 
     if (user->state_validity_ret == FCS_STATE_VALIDITY__EMPTY_SLOT)
     {
@@ -1759,6 +1762,13 @@ char * freecell_solver_user_get_invalid_state_error_string(
            (user->state_validity_ret == FCS_STATE_VALIDITY__MISSING_CARD)
           )
     {
+        /*
+         * user->state_validity_card is only valid for these states,
+         * so we should call fc_solve_card_perl2user on there only.
+         * */
+        char card_str[10];
+        fc_solve_card_perl2user(user->state_validity_card, card_str, print_ts);
+
         sprintf(string, "%s%s.",
             ((user->state_validity_ret == FCS_STATE_VALIDITY__EXTRA_CARD)? "There's an extra card: " : "There's a missing card: "),
             card_str
@@ -1984,7 +1994,6 @@ DLLEXPORT char * freecell_solver_user_iter_state_as_string(
     return
         fc_solve_state_as_string(
             ((fcs_standalone_state_ptrs_t *)ptr_state_void)->key,
-            ((fcs_standalone_state_ptrs_t *)ptr_state_void)->val,
             &(((fcs_standalone_state_ptrs_t *)ptr_state_void)->locs),
             INSTANCE_FREECELLS_NUM,
             INSTANCE_STACKS_NUM,
@@ -2012,7 +2021,7 @@ fcs_int_limit_t DLLEXPORT freecell_solver_user_get_num_states_in_collection_long
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    return user->iterations_board_started_at.num_states_in_collection + user->active_flare->obj_num_states_in_collection - user->init_num_times.num_states_in_collection;
+    return user->iterations_board_started_at.num_states_in_collection + user->active_flare->obj_stats.num_states_in_collection - user->init_num_checked_states.num_states_in_collection;
 }
 
 int DLLEXPORT freecell_solver_user_get_num_states_in_collection(void * api_instance)
@@ -2094,12 +2103,12 @@ int DLLEXPORT freecell_solver_user_next_soft_thread(
 
 extern void DLLEXPORT freecell_solver_user_set_soft_thread_step(
     void * api_instance,
-    int num_times_step
+    int num_checked_states_step
     )
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    user->soft_thread->num_times_step = num_times_step;
+    user->soft_thread->num_checked_states_step = num_checked_states_step;
 }
 
 int DLLEXPORT freecell_solver_user_next_hard_thread(
@@ -2108,7 +2117,7 @@ int DLLEXPORT freecell_solver_user_next_hard_thread(
 {
     fcs_user_t * const user = (fcs_user_t *)api_instance;
 
-    fc_solve_soft_thread_t * soft_thread = fc_solve_new_hard_thread(user->active_flare->obj);
+    fc_solve_soft_thread_t * const soft_thread = fc_solve_new_hard_thread(user->active_flare->obj);
 
     if (soft_thread == NULL)
     {
@@ -2232,7 +2241,7 @@ void DLLEXPORT freecell_solver_user_recycle(
 #if 0
     user->current_iterations_limit = -1;
 #endif
-    user->iterations_board_started_at.num_times = 0;
+    user->iterations_board_started_at.num_checked_states = 0;
     user->iterations_board_started_at.num_states_in_collection = 0;
     if (user->state_string_copy != NULL)
     {
@@ -2362,8 +2371,7 @@ static int user_next_flare(fcs_user_t * user)
     flare->name = NULL;
     flare->fc_pro_moves.moves = NULL;
     flare->instance_is_ready = TRUE;
-    flare->obj_num_times = 0;
-    flare->obj_num_states_in_collection = 0;
+    init_stats(&flare->obj_stats);
 
     return 0;
 }
