@@ -60,7 +60,7 @@ static void my_iter_handler(
     fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
     fprintf(stdout, "Stored-States: %li\n",
-        freecell_solver_user_get_num_states_in_collection_long(user_instance)
+        (long)freecell_solver_user_get_num_states_in_collection_long(user_instance)
         );
     fprintf(stdout, "Scan: %s\n",
         freecell_solver_user_get_current_soft_thread_name(user_instance)
@@ -555,7 +555,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
         fprintf(
             stderr,
             "The number of iterations is %li\n",
-            freecell_solver_user_get_num_times_long(current_instance)
+            (long)freecell_solver_user_get_num_times_long(current_instance)
             );
     }
     else if (command_num == 1)
@@ -618,7 +618,6 @@ static freecell_solver_str_t known_parameters[] = {
 int main(int argc, char * argv[])
 {
     int parser_ret;
-    void * instance;
     int arg;
     FILE * file;
     char user_state[USER_STATE_SIZE];
@@ -630,7 +629,7 @@ int main(int argc, char * argv[])
 
     dc = &debug_context;
 
-    instance = freecell_solver_user_alloc();
+    void * const instance = freecell_solver_user_alloc();
 
     current_instance = instance;
 
@@ -680,7 +679,8 @@ int main(int argc, char * argv[])
         {
             fprintf(stderr, "%s",
                     "Reading the board from the standard input.\n"
-                    "Type \"fc-solve --help\" for more usage information.\n"
+                    "Please refer to the documentation for more usage information:\n"
+                    "    http://fc-solve.shlomifish.org/docs/\n"
                     "To cancel this message set the FREECELL_SOLVER_QUIET environment variable.\n"
                    );
         }
