@@ -43,27 +43,27 @@
 
 static void my_iter_handler(
     void * user_instance,
-    int iter_num,
+    fcs_int_limit_t iter_num,
     int depth,
     void * ptr_state,
-    int parent_iter_num,
+    fcs_int_limit_t parent_iter_num,
     void * lp_context
     )
 {
     fc_solve_display_information_context_t * context;
     context = (fc_solve_display_information_context_t*)lp_context;
 
-    fprintf(stdout, "Iteration: %i\n", iter_num);
+    fprintf(stdout, "Iteration: %li\n", (long)iter_num);
     fprintf(stdout, "Depth: %i\n", depth);
-    fprintf(stdout, "Stored-States: %i\n",
-        freecell_solver_user_get_num_states_in_collection(user_instance)
+    fprintf(stdout, "Stored-States: %li\n",
+        (long)freecell_solver_user_get_num_states_in_collection_long(user_instance)
         );
     fprintf(stdout, "Scan: %s\n",
         freecell_solver_user_get_current_soft_thread_name(user_instance)
         );
     if (context->display_parent_iter_num)
     {
-        fprintf(stdout, "Parent Iteration: %i\n", parent_iter_num);
+        fprintf(stdout, "Parent Iteration: %li\n", (long)parent_iter_num);
     }
     fprintf(stdout, "\n");
 
@@ -108,7 +108,7 @@ static void my_iter_handler(
 
             freecell_solver_user_set_num_freecells(pruner, 2);
             freecell_solver_user_set_tests_order(pruner, "01ABCDE", &error_string);
-            freecell_solver_user_limit_iterations(pruner, 128*1024);
+            freecell_solver_user_limit_iterations_long(pruner, 128*1024);
 
             ret = freecell_solver_user_solve_board(pruner, state_string);
             if (ret == FCS_STATE_SUSPEND_PROCESS)
@@ -491,7 +491,7 @@ static int cmd_line_callback(
     else if ((!strcmp(argv[arg], "-i")) || (!strcmp(argv[arg], "--iter-output")))
     {
 #define set_iter_handler() \
-        freecell_solver_user_set_iter_handler(   \
+        freecell_solver_user_set_iter_handler_long(   \
             instance,   \
             my_iter_handler,   \
             dc    \
@@ -555,7 +555,7 @@ static int cmd_line_callback(
     else if ((!strcmp(argv[arg], "--reset")))
     {
         init_debug_context(dc);
-        freecell_solver_user_set_iter_handler(
+        freecell_solver_user_set_iter_handler_long(
             instance,
             NULL,
             NULL
@@ -591,15 +591,15 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
     {
         fprintf(
             stderr,
-            "The number of iterations is %i\n",
-            freecell_solver_user_get_num_times(current_instance)
+            "The number of iterations is %li\n",
+            (long)freecell_solver_user_get_num_times_long(current_instance)
             );
     }
     else if (command_num == 1)
     {
         if (debug_iter_output_on)
         {
-            freecell_solver_user_set_iter_handler(
+            freecell_solver_user_set_iter_handler_long(
                 current_instance,
                 NULL,
                 NULL
@@ -608,7 +608,7 @@ static void command_signal_handler(int signal_num GCC_UNUSED)
         }
         else
         {
-            freecell_solver_user_set_iter_handler(
+            freecell_solver_user_set_iter_handler_long(
                 current_instance,
                 my_iter_handler,
                 dc
@@ -755,20 +755,20 @@ int main(int argc, char * argv[])
 
 #if 0
     {
-        int limit = 500;
-        freecell_solver_user_limit_iterations(instance, limit);
+        fcs_int_limit_t limit = 500;
+        freecell_solver_user_limit_iterations_long(instance, limit);
         ret = freecell_solver_user_solve_board(instance, user_state);
         while (ret == FCS_STATE_SUSPEND_PROCESS)
         {
             limit += 500;
-            freecell_solver_user_limit_iterations(instance, limit);
+            freecell_solver_user_limit_iterations_long(instance, limit);
             ret = freecell_solver_user_resume_solution(instance);
         }
     }
 #elif defined(FCS_TRACE_MEM)
     {
 #define STEP 100000
-        int limit = STEP;
+        fcs_int_limit_t limit = STEP;
         char stat_fn[1024], foo_str[1024];
         fcs_portable_time_t mytime;
         FILE * stat;
@@ -777,7 +777,7 @@ int main(int argc, char * argv[])
 
         snprintf(stat_fn, sizeof(stat_fn), "/proc/%ld/stat", (long)(getpid()));
 
-        freecell_solver_user_limit_iterations(instance, limit);
+        freecell_solver_user_limit_iterations_long(instance, limit);
         ret = freecell_solver_user_solve_board(instance, user_state);
         while (ret == FCS_STATE_SUSPEND_PROCESS)
         {
@@ -832,7 +832,7 @@ int main(int argc, char * argv[])
 
             fflush(stdout);
             limit += STEP;
-            freecell_solver_user_limit_iterations(instance, limit);
+            freecell_solver_user_limit_iterations_long(instance, limit);
             ret = freecell_solver_user_resume_solution(instance);
         }
     }
