@@ -14,6 +14,7 @@ use File::Path;
 use Getopt::Long;
 use Env::Path;
 use File::Basename qw( basename dirname );
+# use Time::HiRes qw(time);
 
 
 my $bindir = dirname( __FILE__ );
@@ -40,7 +41,7 @@ sub run_tests
     exec(($use_prove ? @{_calc_prove()} : 'runprove'), @$tests);
 }
 
-my $tests_glob = "*.{exe,py,t}";
+my $tests_glob = "*.{t.exe,py,t}";
 
 GetOptions(
     '--glob=s' => \$tests_glob,
@@ -164,10 +165,14 @@ GetOptions(
         }
     }
 
+    # Cancelled because it is done by the build system.
     # if (! glob('t/valgrind--*.t'))
-    {
-        system($^X, "$abs_bindir/scripts/generate-individual-valgrind-test-scripts.pl");
-    }
+    # {
+        # my $start = time();
+        # system($^X, "$abs_bindir/scripts/generate-individual-valgrind-test-scripts.pl");
+        # my $end = time();
+        # print "StartGen=$start\nEndGen=$end\n";
+    # }
 
     # Put the valgrind tests last, because they take a long time.
     my @tests =
@@ -183,7 +188,7 @@ GetOptions(
                 ||
             ($a cmp $b)
         }
-        (glob("t/$tests_glob"),
+        (glob("t/$tests_glob"),glob("./$tests_glob"),
             (
                 ($fcs_path ne $abs_bindir)
                 ? (glob("$abs_bindir/t/t/$tests_glob"))
