@@ -39,7 +39,6 @@
 #include "config.h"
 
 #include "state.h"
-#include "card.h"
 #include "scans.h"
 #include "meta_alloc.h"
 #include "check_and_add_state.h"
@@ -52,6 +51,7 @@
 #include "inline.h"
 #include "likely.h"
 #include "bool.h"
+#include "min_and_max.h"
 
 #if 0
 #define DEBUG 1
@@ -61,21 +61,30 @@
 #undef DEBUG
 #endif
 
-#ifdef FCS_WITHOUT_DEPTH_FIELD
-static GCC_INLINE int calc_depth(fcs_collectible_state_t * ptr_state)
+typedef int fcs_depth_t;
+
+static GCC_INLINE fcs_depth_t calc_depth(fcs_collectible_state_t * ptr_state)
 {
-    register int ret = 0;
+#ifdef FCS_WITHOUT_DEPTH_FIELD
+
+    register fcs_depth_t ret = 0;
     while ((ptr_state = FCS_S_PARENT(ptr_state)) != NULL)
     {
         ret++;
     }
     return ret;
-}
-#else
-#define calc_depth(ptr_state) (FCS_S_DEPTH(ptr_state))
-#endif
 
-#define kv_calc_depth(ptr_state) (calc_depth(FCS_STATE_kv_to_collectible(ptr_state)))
+#else
+
+    return (FCS_S_DEPTH(ptr_state));
+
+#endif
+}
+
+static GCC_INLINE fcs_depth_t kv_calc_depth(fcs_kv_state_t * ptr_state)
+{
+    return calc_depth(FCS_STATE_kv_to_collectible(ptr_state));
+}
 
 #define SOFT_DFS_DEPTH_GROW_BY 16
 void fc_solve_increase_dfs_max_depth(
@@ -334,7 +343,7 @@ static GCC_INLINE pq_rating_t befs_rate_state(
 #endif
 
 #ifndef FCS_FREECELL_ONLY
-    fcs_bool_t int_unlimited_sequence_move = INSTANCE_UNLIMITED_SEQUENCE_MOVE;
+    const fcs_bool_t int_unlimited_sequence_move = INSTANCE_UNLIMITED_SEQUENCE_MOVE;
 #define unlimited_sequence_move int_unlimited_sequence_move
 #else
 #define unlimited_sequence_move FALSE
@@ -359,7 +368,7 @@ static GCC_INLINE pq_rating_t befs_rate_state(
     {
         for (int a = 0 ; a < LOCAL_STACKS_NUM ; a++)
         {
-            const fcs_cards_column_t col = fcs_state_get_col(*state, a);
+            const fcs_const_cards_column_t col = fcs_state_get_col(*state, a);
             const int cards_num = fcs_col_len(col);
 
             if (cards_num <= 1)
@@ -430,7 +439,7 @@ static GCC_INLINE pq_rating_t befs_rate_state(
         int num_cards_not_on_parents = (LOCAL_DECKS_NUM*52);
         for (int stack_idx = 0 ; stack_idx < LOCAL_STACKS_NUM ; stack_idx++)
         {
-            const fcs_cards_column_t col =
+            const fcs_const_cards_column_t col =
                 fcs_state_get_col(*state, stack_idx);
 
             const int col_len = fcs_col_len(col);
@@ -494,7 +503,7 @@ static GCC_INLINE pq_rating_t befs_rate_state(
                     message, \
                     (long int)DFS_VAR(soft_thread, depth), \
                     (long int)(the_soft_dfs_info-DFS_VAR(soft_thread, soft_dfs_info)), \
-                    (long int)(instance->num_times), \
+                    (long int)(instance->num_checked_states), \
                     the_soft_dfs_info->tests_list_index, \
                     the_soft_dfs_info->test_index, \
                     the_soft_dfs_info->current_state_index, \
@@ -611,10 +620,10 @@ static GCC_INLINE void mark_as_dead_end(
     return;
 }
 
-#define BUMP_NUM_TIMES() \
+#define BUMP_NUM_CHECKED_STATES() \
 {       \
-    (*instance_num_times_ptr)++; \
-    (*hard_thread_num_times_ptr)++; \
+    (*instance_num_checked_states_ptr)++; \
+    (*hard_thread_num_checked_states_ptr)++; \
 }
 
 #define SHOULD_STATE_BE_PRUNED(enable_pruning, ptr_state) \
@@ -988,7 +997,7 @@ int fc_solve_soft_dfs_do_solve(
     const int soft_thread_id = soft_thread->id;
     const fcs_tests_list_of_lists * the_tests_list_ptr;
     fcs_tests_group_type_t local_shuffling_type = FCS_NO_SHUFFLING;
-    fcs_int_limit_t hard_thread_max_num_times;
+    fcs_int_limit_t hard_thread_max_num_checked_states;
 
 #if ((!defined(HARD_CODED_NUM_FREECELLS)) || (!defined(HARD_CODED_NUM_STACKS)))
     SET_GAME_PARAMS();
@@ -1024,22 +1033,22 @@ int fc_solve_soft_dfs_do_solve(
     }
 
 
-    fcs_int_limit_t * const instance_num_times_ptr = &(instance->num_times);
-    fcs_int_limit_t * const hard_thread_num_times_ptr
-        = &(hard_thread->num_times);
+    fcs_int_limit_t * const instance_num_checked_states_ptr = &(instance->num_checked_states);
+    fcs_int_limit_t * const hard_thread_num_checked_states_ptr
+        = &(hard_thread->num_checked_states);
 
-#define CALC_HARD_THREAD_MAX_NUM_TIMES() \
-    hard_thread_max_num_times = hard_thread->max_num_times; \
+#define CALC_HARD_THREAD_MAX_NUM_CHECKED_STATES() \
+    hard_thread_max_num_checked_states = hard_thread->max_num_checked_states; \
                 \
     {           \
-        fcs_int_limit_t lim = hard_thread->num_times       \
-            + (instance->effective_max_num_times - *(instance_num_times_ptr)) \
+        fcs_int_limit_t lim = hard_thread->num_checked_states       \
+            + (instance->effective_max_num_checked_states - *(instance_num_checked_states_ptr)) \
             ; \
               \
-        hard_thread_max_num_times = min(hard_thread_max_num_times, lim); \
+        hard_thread_max_num_checked_states = min(hard_thread_max_num_checked_states, lim); \
     }
 
-    CALC_HARD_THREAD_MAX_NUM_TIMES();
+    CALC_HARD_THREAD_MAX_NUM_CHECKED_STATES();
 
     const fcs_instance_debug_iter_output_func_t debug_iter_output_func = instance->debug_iter_output_func;
     const fcs_instance_debug_iter_output_context_t debug_iter_output_context = instance->debug_iter_output_context;
@@ -1146,7 +1155,7 @@ int fc_solve_soft_dfs_do_solve(
                 {
                     debug_iter_output_func(
                         debug_iter_output_context,
-                        *(instance_num_times_ptr),
+                        *(instance_num_checked_states_ptr),
                         DEPTH(),
                         (void*)instance,
                         STATE_TO_PASS(),
@@ -1173,7 +1182,7 @@ int fc_solve_soft_dfs_do_solve(
                 {
                     instance->final_state = PTR_STATE;
 
-                    BUMP_NUM_TIMES();
+                    BUMP_NUM_CHECKED_STATES();
 
                     TRACE0("Returning FCS_STATE_WAS_SOLVED");
                     return FCS_STATE_WAS_SOLVED;
@@ -1409,7 +1418,7 @@ int fc_solve_soft_dfs_do_solve(
                     )
                    )
                 {
-                    BUMP_NUM_TIMES();
+                    BUMP_NUM_CHECKED_STATES();
 
                     VERIFY_PTR_STATE_AND_DERIVED_TRACE0("Verify Gypsy");
 
@@ -1419,7 +1428,7 @@ int fc_solve_soft_dfs_do_solve(
                     );
 
 #ifndef FCS_WITHOUT_VISITED_ITER
-                    FCS_S_VISITED_ITER(single_derived_state) = instance->num_times;
+                    FCS_S_VISITED_ITER(single_derived_state) = instance->num_checked_states;
 #endif
 
                     VERIFY_PTR_STATE_AND_DERIVED_TRACE0("Verify Golf");
@@ -1474,7 +1483,7 @@ int fc_solve_soft_dfs_do_solve(
      * We need to bump the number of iterations so it will be ready with
      * a fresh iterations number for the next scan that takes place.
      * */
-    BUMP_NUM_TIMES();
+    BUMP_NUM_CHECKED_STATES();
 
     DEPTH() = -1;
 
@@ -1504,7 +1513,7 @@ int fc_solve_soft_dfs_do_solve(
             { \
             printf("BestFS - %s ; Iters=%ld.\n", \
                     message, \
-                    (long)(instance->num_times) \
+                    (long)(instance->num_checked_states) \
                     );  \
             fflush(stdout); \
             } \
@@ -1686,7 +1695,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
 
     int error_code;
 
-    fcs_int_limit_t hard_thread_max_num_times;
+    fcs_int_limit_t hard_thread_max_num_checked_states;
 
     fcs_derived_states_list_t derived;
     derived.num_states = 0;
@@ -1701,8 +1710,8 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
     const fcs_bool_t enable_pruning = soft_thread->enable_pruning;
 
     const int method = soft_thread->method;
-    fcs_int_limit_t * const instance_num_times_ptr = &(instance->num_times);
-    fcs_int_limit_t * const hard_thread_num_times_ptr = &(hard_thread->num_times);
+    fcs_int_limit_t * const instance_num_checked_states_ptr = &(instance->num_checked_states);
+    fcs_int_limit_t * const hard_thread_num_checked_states_ptr = &(hard_thread->num_checked_states);
 
     INITIALIZE_STATE();
 
@@ -1720,7 +1729,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
     SET_GAME_PARAMS();
 #endif
 
-    CALC_HARD_THREAD_MAX_NUM_TIMES();
+    CALC_HARD_THREAD_MAX_NUM_CHECKED_STATES();
 
     const fcs_instance_debug_iter_output_func_t debug_iter_output_func = instance->debug_iter_output_func;
     const fcs_instance_debug_iter_output_context_t debug_iter_output_context = instance->debug_iter_output_context;
@@ -1811,7 +1820,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
         {
             debug_iter_output_func(
                     debug_iter_output_context,
-                    *(instance_num_times_ptr),
+                    *(instance_num_checked_states_ptr),
                     calc_depth(PTR_STATE),
                     (void*)instance,
                     STATE_TO_PASS(),
@@ -1831,7 +1840,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
         {
             instance->final_state = PTR_STATE;
 
-            BUMP_NUM_TIMES();
+            BUMP_NUM_CHECKED_STATES();
 
             error_code = FCS_STATE_WAS_SOLVED;
             goto my_return_label;
@@ -1874,7 +1883,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
 
         /* Increase the number of iterations by one .
          * */
-        BUMP_NUM_TIMES();
+        BUMP_NUM_CHECKED_STATES();
 
 
         TRACE0("Insert all states");
@@ -1957,7 +1966,7 @@ int fc_solve_befs_or_bfs_do_solve( fc_solve_soft_thread_t * const soft_thread )
         }
 
 #ifndef FCS_WITHOUT_VISITED_ITER
-        FCS_S_VISITED_ITER(PTR_STATE) = *(instance_num_times_ptr)-1;
+        FCS_S_VISITED_ITER(PTR_STATE) = *(instance_num_checked_states_ptr)-1;
 #endif
 
 label_next_state:
@@ -2047,6 +2056,31 @@ static GCC_INLINE char * * fc_solve_calc_positions_by_rank_location(
     }
 }
 
+static GCC_INLINE void assign_dest_stack_and_col_ptr(
+    char * const positions_by_rank,
+    int dest_stack,
+    int dest_col,
+    fcs_card_t dest_card
+)
+{
+    char * ptr = &positions_by_rank[
+        (FCS_POS_BY_RANK_WIDTH *
+         (fcs_card_rank(dest_card)-1)
+        )
+        +
+        (fcs_card_suit(dest_card)<<1)
+        ];
+
+#if (!defined(HARD_CODED_NUM_DECKS) || (HARD_CODED_NUM_DECKS == 1))
+    for(;(*ptr) != -1;ptr += (4 << 1))
+    {
+    }
+#endif
+
+    *(ptr++) = (char)dest_stack;
+    *(ptr) = (char)dest_col;
+}
+
 char * fc_solve_get_the_positions_by_rank_data__freecell_generator(
     fc_solve_soft_thread_t * const soft_thread,
     const fcs_state_t * const ptr_state_key
@@ -2085,64 +2119,47 @@ char * fc_solve_get_the_positions_by_rank_data__freecell_generator(
         /* Populate positions_by_rank by looping over the stacks and
          * indices looking for the cards and filling them. */
 
+        for (int ds = 0 ; ds < LOCAL_STACKS_NUM ; ds++)
         {
-            char * ptr;
+            const fcs_const_cards_column_t dest_col = fcs_state_get_col(the_state, ds);
+            int top_card_idx = fcs_col_len(dest_col);
 
-            for (int ds = 0 ; ds < LOCAL_STACKS_NUM ; ds++)
+            if (unlikely((top_card_idx--) == 0))
             {
-                const fcs_cards_column_t dest_col = fcs_state_get_col(the_state, ds);
-                int top_card_idx = fcs_col_len(dest_col);
+                continue;
+            }
 
-                if (unlikely((top_card_idx--) == 0))
+            fcs_card_t dest_card;
+            {
+                fcs_card_t dest_below_card;
+                dest_card = fcs_col_get_card(dest_col, 0);
+                for (
+                    int dc=0
+                    ;
+                    dc < top_card_idx
+                    ;
+                    dc++,
+                    dest_card = dest_below_card
+                )
                 {
-                    continue;
-                }
-
-                fcs_card_t dest_card;
-                {
-                    fcs_card_t dest_below_card;
-                    int dc;
-                    for (
-                        dc=0,
-                        dest_card = fcs_col_get_card(dest_col, 0)
-                        ;
-                        dc < top_card_idx
-                        ;
-                        dc++,
-                        dest_card = dest_below_card
-                    )
+                    dest_below_card = fcs_col_get_card(dest_col, dc+1);
+                    if (!fcs_is_parent_card(dest_below_card, dest_card))
                     {
-                        dest_below_card = fcs_col_get_card(dest_col, dc+1);
-                        if (!fcs_is_parent_card(dest_below_card, dest_card))
-                        {
-#if (!defined(HARD_CODED_NUM_DECKS) || (HARD_CODED_NUM_DECKS == 1))
-#define INCREMENT_PTR_BY_NUM_DECKS() \
-                            for(;(*ptr) != -1;ptr += (4 << 1)) \
-                            { \
-                            }
-#else
-#define INCREMENT_PTR_BY_NUM_DECKS() \ {}
-#endif
-
-#define ASSIGN_PTR(dest_stack, dest_col) \
-                            ptr = &positions_by_rank[ \
-                            (FCS_POS_BY_RANK_WIDTH * \
-                             (fcs_card_rank(dest_card)-1) \
-                            ) \
-                            + \
-                            (fcs_card_suit(dest_card)<<1) \
-                            ]; \
-                            INCREMENT_PTR_BY_NUM_DECKS() \
-                            *(ptr++) = (char)(dest_stack); \
-                            *(ptr) = (char)(dest_col)
-
-
-                            ASSIGN_PTR(ds, dc);
-                        }
+                        assign_dest_stack_and_col_ptr(
+                            positions_by_rank,
+                            ds,
+                            dc,
+                            dest_card
+                        );
                     }
                 }
-                ASSIGN_PTR(ds, top_card_idx);
             }
+            assign_dest_stack_and_col_ptr(
+                positions_by_rank,
+                ds,
+                top_card_idx,
+                dest_card
+            );
         }
     }
 
