@@ -22,11 +22,14 @@ fi
 
 max_iters="${MAX_ITERS:-1500000}"
 
+export FREECELL_SOLVER_QUIET=1
+
 seq "$start" "$end" |  \
 (while read deal ; do
     echo "${deal}:"
     ./board_gen/make_pysol_freecell_board.py $ms -t "$deal" "$variant" |
         ./fc-solve --game "$variant" $theme -sel -sam -p -t -mi "$max_iters" |
-        tail -3
+        tail -3 ;
+    date +"T=%s.%N"
 done) | tee -a total_dump.txt
 
