@@ -34,7 +34,6 @@
 
 #include "config.h"
 #include "state.h"
-#include "card.h"
 #include "fcs_enums.h"
 #include "app_str.h"
 #include "unused.h"
@@ -43,94 +42,33 @@
 #ifdef DEBUG_STATES
 static GCC_INLINE int fcs_stack_compare(const void * s1, const void * s2)
 {
-    fcs_card_t card1 = ((const fc_stack_t *)s1)->cards[0];
-    fcs_card_t card2 = ((const fc_stack_t *)s2)->cards[0];
-
-    return fc_solve_card_compare(&card1, &card2);
+#define GET_CARD(s) (((const fc_stack_t *)(s))->cards[0])
+    return fc_solve_card_compare(GET_CARD(s1), GET_CARD(s2));
+#undef GET_CARD
 }
 #elif defined(COMPACT_STATES)
 static GCC_INLINE int fcs_stack_compare(const void * s1, const void * s2)
 {
-    fcs_card_t card1 = ((const fcs_card_t*)s1)[1];
-    fcs_card_t card2 = ((const fcs_card_t*)s2)[1];
-
-    return fc_solve_card_compare(&card1, &card2);
+#define GET_CARD(s) (((const fcs_card_t *)(s))[1])
+    return fc_solve_card_compare(GET_CARD(s1), GET_CARD(s2));
+#undef GET_CARD
 }
 #endif
 
-#ifdef DEBUG_STATES
-void fc_solve_canonize_state(
-    fcs_collectible_state_t * state,
-    int freecells_num,
-    int stacks_num)
-{
-    int b,c;
-
-    fc_stack_t temp_stack;
-    fcs_card_t temp_freecell;
-    int temp_loc;
-
-    /* Insertion-sort the stacks */
-    for(b=1;b<stacks_num;b++)
-    {
-        c = b;
-        while(
-            (c>0) &&
-            (fcs_stack_compare(
-                &(state_key->stacks[c]),
-                &(state_key->stacks[c-1])
-                ) < 0)
-            )
-        {
-            temp_stack = state_key->stacks[c];
-            state_key->stacks[c] = state_key->stacks[c-1];
-            state_key->stacks[c-1] = temp_stack;
-
-            temp_loc = state_val->stack_locs[c];
-            state_val->stack_locs[c] = state_val->stack_locs[c-1];
-            state_val->stack_locs[c-1] = temp_loc;
-
-            c--;
-        }
-    }
-
-    /* Insertion sort the freecells */
-
-    for(b=1;b<freecells_num;b++)
-    {
-        c = b;
-        while(
-            (c>0)     &&
-            (fc_solve_card_compare(
-                &(state_key->freecells[c]),
-                &(state_key->freecells[c-1])
-                ) < 0)
-            )
-        {
-            temp_freecell = state_key->freecells[c];
-            state_key->freecells[c] = state_key->freecells[c-1];
-            state_key->freecells[c-1] = temp_freecell;
-
-            temp_loc = state_val->fc_locs[c];
-            state_val->fc_locs[c] = state_val->fc_locs[c-1];
-            state_val->fc_locs[c-1] = temp_loc;
-
-            c--;
-        }
-    }
-}
-
-#elif defined(COMPACT_STATES) || defined(INDIRECT_STACK_STATES)
-
 #ifdef COMPACT_STATES
 #define DECLARE_TEMP_STACK() char temp_stack[(MAX_NUM_CARDS_IN_A_STACK+1)]
 #define STACK_COMPARE(a,b) (fcs_stack_compare((a),(b)))
 #define GET_STACK(c) (state_key->data+(c)*(MAX_NUM_CARDS_IN_A_STACK+1))
 #define COPY_STACK(d,s) (memcpy(d, s, (MAX_NUM_CARDS_IN_A_STACK+1)))
 #define GET_FREECELL(c) (state_key->data[FCS_FREECELLS_OFFSET+(c)])
-#elif defined(INDIRECT_STACK_STATES)
+#elif defined(DEBUG_STATES) || defined(INDIRECT_STACK_STATES)
+#ifdef DEBUG_STATES
+#define DECLARE_TEMP_STACK() fc_stack_t temp_stack
+#define STACK_COMPARE(a,b) (fcs_stack_compare((&(a)),(&(b))))
+#else
 #define DECLARE_TEMP_STACK() fcs_card_t * temp_stack
-#define STACK_COMPARE(a,b) (fc_solve_stack_compare_for_comparison((a),(b)))
+#define STACK_COMPARE(a,b) (fc_solve_stack_compare_for_comparison(a,b))
+#endif
 #define GET_STACK(c) (state_key->stacks[c])
 #define COPY_STACK(d,s) (d = s)
 #define GET_FREECELL(c) (state_key->freecells[(c)])
@@ -179,8 +117,8 @@ void fc_solve_canonize_state(
         while(
             (c>0)    &&
             ((fc_solve_card_compare(
-                &(GET_FREECELL(c)),
-                &(GET_FREECELL(c-1))
+                (GET_FREECELL(c)),
+                (GET_FREECELL(c-1))
             )
             ) < 0)
         )
@@ -199,7 +137,6 @@ void fc_solve_canonize_state(
 void fc_solve_canonize_state_with_locs(
     fcs_kv_state_t * state,
 #define state_key (state->key)
-#define state_val (state->val)
     fcs_state_locs_struct_t * locs,
     int freecells_num,
     int stacks_num)
@@ -246,8 +183,8 @@ void fc_solve_canonize_state_with_locs(
         while(
             (c>0)    &&
             ((fc_solve_card_compare(
-                &(GET_FREECELL(c)),
-                &(GET_FREECELL(c-1))
+                (GET_FREECELL(c)),
+                (GET_FREECELL(c-1))
             )
             ) < 0)
         )
@@ -265,11 +202,7 @@ void fc_solve_canonize_state_with_locs(
     }
 }
 #undef state_key
-#undef state_val
 
-#elif 0
-
-#endif
 
 #if (FCS_STATE_STORAGE != FCS_STATE_STORAGE_INDIRECT)
 
@@ -305,10 +238,9 @@ int fc_solve_state_compare_indirect_with_context(const void * s1, const void * s
 
 char * fc_solve_state_as_string(
     fcs_state_t * state,
-    fcs_state_extra_info_t * state_val,
     fcs_state_locs_struct_t * state_locs,
-#define FCS_S_STACK_LOCS(s) (state_locs->stack_locs)
-#define FCS_S_FC_LOCS(s) (state_locs->fc_locs)
+#define FCS_S_STACK_LOCS() (state_locs->stack_locs)
+#define FCS_S_FC_LOCS() (state_locs->fc_locs)
     int freecells_num,
     int stacks_num,
     int decks_num,
@@ -347,11 +279,11 @@ char * fc_solve_state_as_string(
     {
         for(a=0;a<stacks_num;a++)
         {
-            stack_locs[(int)((FCS_S_STACK_LOCS(state_val))[a])] = a;
+            stack_locs[(int)((FCS_S_STACK_LOCS())[a])] = a;
         }
         for(a=0;a<freecells_num;a++)
         {
-            freecell_locs[(int)((FCS_S_FC_LOCS(state_val))[a])] = a;
+            freecell_locs[(int)((FCS_S_FC_LOCS())[a])] = a;
         }
     }
 
