@@ -41,8 +41,9 @@ extern "C" {
 
 #include "inline.h"
 #include "bool.h"
+#include "min_and_max.h"
 
-#include "card.h"
+#include "game_type_limit.h"
 
 #include "internal_move_struct.h"
 #include "indirect_buffer.h"
@@ -83,6 +84,7 @@ typedef struct
 } fc_stack_t;
 
 typedef fc_stack_t * fcs_cards_column_t;
+typedef const fc_stack_t * fcs_const_cards_column_t;
 typedef int fcs_state_foundation_t;
 
 struct fcs_struct_state_t
@@ -102,6 +104,14 @@ static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
 
     return ret;
 }
+static GCC_INLINE char fcs_card2char(const fcs_card_t card)
+{
+    return (char)(card.suit | (card.rank << 2));
+}
+static GCC_INLINE fcs_card_t fcs_char2card(unsigned char c)
+{
+    return fcs_make_card((c >> 2), (c & 0x3));
+}
 
 #define fcs_state_get_col(state, col_idx) \
     (&((state).stacks[(col_idx)]))
@@ -145,6 +155,7 @@ static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
 
 typedef char fcs_card_t;
 typedef fcs_card_t * fcs_cards_column_t;
+typedef const fcs_card_t * fcs_const_cards_column_t;
 typedef fcs_card_t fcs_state_foundation_t;
 /*
  * Card:
@@ -202,6 +213,7 @@ typedef char fcs_locs_t;
 
 typedef char fcs_card_t;
 typedef fcs_card_t * fcs_cards_column_t;
+typedef const fcs_card_t * fcs_const_cards_column_t;
 typedef char fcs_state_foundation_t;
 
 struct fcs_struct_state_t
@@ -357,6 +369,9 @@ static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
     return ( (((fcs_card_t)rank) << 2) | ((fcs_card_t)suit) );
 }
 
+#define fcs_card2char(c) (c)
+#define fcs_char2card(c) (c)
+
 #define fcs_col_len(col) \
     ( ((col)[0]) )
 
@@ -387,9 +402,6 @@ static GCC_INLINE fcs_card_t fcs_make_card(const int rank, const int suit)
 
 #endif
 
-
-typedef unsigned char fcs_game_limit_t;
-
 struct fcs_state_keyval_pair_struct;
 
 /*
@@ -604,7 +616,20 @@ typedef struct {
     fcs_state_locs_struct_t locs;
 } fcs_standalone_state_ptrs_t;
 
+#ifdef DEBUG_STATES
+
 extern fcs_card_t fc_solve_empty_card;
+#define DEFINE_fc_solve_empty_card() \
+    fcs_card_t fc_solve_empty_card = {0,0};
+
+#elif defined(COMPACT_STATES) || defined (INDIRECT_STACK_STATES)
+
+#define fc_solve_empty_card ((fcs_card_t)0)
+
+#define DEFINE_fc_solve_empty_card() \
+    ;
+
+#endif
 
 extern void fc_solve_canonize_state(
     fcs_kv_state_t * state_raw,
@@ -662,7 +687,7 @@ extern fcs_card_t fc_solve_card_user2perl(const char * str);
 extern char * fc_solve_card_perl2user(
     fcs_card_t card,
     char * str,
-    int t
+    fcs_bool_t t
     );
 
 /*
@@ -678,9 +703,9 @@ extern char * fc_solve_p2u_rank(
     int rank_idx,
     char * str,
     fcs_bool_t * rank_is_null,
-    int t
+    fcs_bool_t t
 #ifndef FCS_WITHOUT_CARD_FLIPPING
-    , int flipped
+    , fcs_bool_t flipped
 #endif
     );
 
@@ -751,9 +776,11 @@ static GCC_INLINE void fc_solve_state_init_proto(
 static const char * const fc_solve_freecells_prefixes[] = { "FC:", "Freecells:", "Freecell:", NULL};
 
 static const char * const fc_solve_foundations_prefixes[] = { "Decks:", "Deck:", "Founds:", "Foundations:", "Foundation:", "Found:", NULL};
-#ifdef WIN32
+#if defined(WIN32) && (!defined(HAVE_STRNCASECMP))
+#ifndef strncasecmp
 #define strncasecmp(a,b,c) (strnicmp((a),(b),(c)))
 #endif
+#endif
 
 #ifdef INDIRECT_STACK_STATES
 #define fc_solve_initial_user_state_to_c(string, out_state, freecells_num, stacks_num, decks_num, indirect_stacks_buffer) \
@@ -800,7 +827,7 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c_proto(
         } \
     }
 
-    for(s=0;s<stacks_num;s++)
+    for (s = 0 ; s < stacks_num ; s++)
     {
         /* Move to the next stack */
         if (!first_line)
@@ -826,11 +853,11 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c_proto(
 
         if (*prefix)
         {
-            for(c=0;c<freecells_num;c++)
+            for (c = 0 ; c < freecells_num ; c++)
             {
                 fcs_empty_freecell(ret, c);
             }
-            for(c=0;c<freecells_num;c++)
+            for (c = 0 ; c < freecells_num ; c++)
             {
                 if (c!=0)
                 {
@@ -988,7 +1015,6 @@ static GCC_INLINE int fc_solve_initial_user_state_to_c_proto(
 
 extern char * fc_solve_state_as_string(
     fcs_state_t * key,
-    fcs_state_extra_info_t * state_val,
     fcs_state_locs_struct_t * state_locs,
     int freecells_num,
     int stacks_num,
@@ -1110,46 +1136,37 @@ enum
     FCS_VISITED_GENERATED_BY_PRUNING = 0x10,
 };
 
-#ifndef min
-#define min(a,b) ((a)<(b)?(a):(b))
-#endif
-
-#ifndef max
-#define max(a,b) ((a)>(b)?(a):(b))
-#endif
-
+#ifndef DEBUG_STATES
 #define FCS_WITH_CARD_COMPARE_LOOKUP
+#endif
 
 static GCC_INLINE int fc_solve_card_compare(
-        const fcs_card_t * const c1,
-        const fcs_card_t * const c2
+        const fcs_card_t c1,
+        const fcs_card_t c2
         )
 {
 #ifdef FCS_WITH_CARD_COMPARE_LOOKUP
-    return (*c1)-(*c2);
+    return (c1)-(c2);
 #else
-    if (fcs_card_rank(*c1) > fcs_card_rank(*c2))
+    if (fcs_card_rank(c1) > fcs_card_rank(c2))
     {
         return 1;
     }
-    else if (fcs_card_rank(*c1) < fcs_card_rank(*c2))
+    else if (fcs_card_rank(c1) < fcs_card_rank(c2))
+    {
+        return -1;
+    }
+    else if (fcs_card_suit(c1) > fcs_card_suit(c2))
+    {
+        return 1;
+    }
+    else if (fcs_card_suit(c1) < fcs_card_suit(c2))
     {
         return -1;
     }
     else
     {
-        if (fcs_card_suit(*c1) > fcs_card_suit(*c2))
-        {
-            return 1;
-        }
-        else if (fcs_card_suit(*c1) < fcs_card_suit(*c2))
-        {
-            return -1;
-        }
-        else
-        {
-            return 0;
-        }
+        return 0;
     }
 #endif
 }
@@ -1162,11 +1179,9 @@ static GCC_INLINE int fc_solve_stack_compare_for_comparison(const void * const v
 
     const int min_len = min(s1[0], s2[0]);
     {
-        int a, ret;
-
-        for(a=0;a<min_len;a++)
+        for(int a=1 ; a <= min_len ; a++)
         {
-            ret = fc_solve_card_compare(s1+a+1,s2+a+1);
+            int ret = fc_solve_card_compare(s1[a],s2[a]);
             if (ret != 0)
             {
                 return ret;
@@ -1196,15 +1211,4 @@ static GCC_INLINE int fc_solve_stack_compare_for_comparison(const void * const v
 
 #endif
 
-#ifdef DEBUG_STATES
-
-#define DEFINE_fc_solve_empty_card() \
-    fcs_card_t fc_solve_empty_card = {0,0};
-
-#elif defined(COMPACT_STATES) || defined (INDIRECT_STACK_STATES)
-
-#define DEFINE_fc_solve_empty_card() \
-    fcs_card_t fc_solve_empty_card = (fcs_card_t)0;
-
-#endif
 #endif /* FC_SOLVE__STATE_H */
