@@ -288,6 +288,14 @@ int main(int argc, char * argv[])
     end_board = atoi(argv[arg++]);
     stop_at = atoi(argv[arg++]);
 
+    if (stop_at <= 0)
+    {
+        fprintf(stderr, "print_step (the third argument) must be greater than 0.\n");
+        print_help();
+        exit(-1);
+
+    }
+
     init_debug_context(&user.display_context);
 
     for (;arg < argc; arg++)
